@@ -1,16 +1,17 @@
 //! Types and traits for converting objects to addresses which
 //! Tor can connect to.
 
-use crate::err::ErrorDetail;
 use crate::StreamPrefs;
+use crate::err::ErrorDetail;
 use std::fmt::Display;
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV6};
 use std::str::FromStr;
 use thiserror::Error;
 use tor_basic_utils::StrExt;
+use tor_error::{ErrorKind, HasKind};
 
 #[cfg(feature = "onion-service-client")]
-use tor_hscrypto::pk::{HsId, HSID_ONION_SUFFIX};
+use tor_hscrypto::pk::{HSID_ONION_SUFFIX, HsId};
 
 /// Fake plastic imitation of some of the `tor-hs*` functionality
 #[cfg(not(feature = "onion-service-client"))]
@@ -230,6 +231,14 @@ impl TorAddr {
         matches!(&self.host, Host::Ip(_))
     }
 
+    /// If this TorAddr is an explicit IP address, return a reference to that [`IpAddr`].
+    pub fn as_ip_address(&self) -> Option<&IpAddr> {
+        match &self.host {
+            Host::Ip(a) => Some(a),
+            _ => None,
+        }
+    }
+
     /// Get instructions for how to make a stream to this address
     pub(crate) fn into_stream_instructions(
         self,
@@ -349,7 +358,6 @@ impl std::fmt::Display for TorAddr {
 // to it, or expose lower-level errors in it, without careful consideration!
 #[derive(Debug, Error, Clone, Eq, PartialEq)]
 #[non_exhaustive]
-// TODO Should implement ErrorKind
 pub enum TorAddrError {
     /// Tried to parse a string that can never be interpreted as a valid host.
     #[error("String can never be a valid hostname")]
@@ -362,6 +370,19 @@ pub enum TorAddrError {
     BadPort,
 }
 
+impl HasKind for TorAddrError {
+    fn kind(&self) -> ErrorKind {
+        use ErrorKind as EK;
+        use TorAddrError as TAE;
+
+        match self {
+            TAE::InvalidHostname => EK::InvalidStreamTarget,
+            TAE::NoPort => EK::InvalidStreamTarget,
+            TAE::BadPort => EK::InvalidStreamTarget,
+        }
+    }
+}
+
 /// A host that Tor can connect to: either a hostname or an IP address.
 //
 // We use `String` in here, and pass that directly to (for example)
@@ -568,6 +589,18 @@ mod test {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
 
+    #[test]
+    fn test_error_kind() {
+        use tor_error::ErrorKind as EK;
+
+        assert_eq!(
+            TorAddrError::InvalidHostname.kind(),
+            EK::InvalidStreamTarget
+        );
+        assert_eq!(TorAddrError::NoPort.kind(), EK::InvalidStreamTarget);
+        assert_eq!(TorAddrError::BadPort.kind(), EK::InvalidStreamTarget);
+    }
+
     /// Make a `StreamPrefs` with `.onion` enabled, if cfg-enabled
     fn mk_stream_prefs() -> StreamPrefs {
         let prefs = crate::StreamPrefs::default();
@@ -764,9 +797,9 @@ mod test {
     #[test]
     fn prefs_onion_services() {
         use crate::err::ErrorDetailDiscriminants;
-        use tor_error::{ErrorKind, HasKind as _};
         use ErrorDetailDiscriminants as EDD;
         use ErrorKind as EK;
+        use tor_error::{ErrorKind, HasKind as _};
 
         #[allow(clippy::redundant_closure)] // for symmetry with prefs_of, below, and clarity
         let prefs_def = || StreamPrefs::default();
@@ -806,8 +839,8 @@ mod test {
                     prefs.connect_to_onion_services(yn);
                     prefs
                 };
-                check_stream(prefs_def(), Err((EDD::OnionAddressDisabled, EK::ForbiddenStreamTarget)));
-                check_stream(prefs_of(B::Auto), Err((EDD::OnionAddressDisabled, EK::ForbiddenStreamTarget)));
+                check_stream(prefs_def(), Ok(()));
+                check_stream(prefs_of(B::Auto), Ok(()));
                 check_stream(prefs_of(B::Explicit(true)), Ok(()));
                 check_stream(prefs_of(B::Explicit(false)), Err((EDD::OnionAddressDisabled, EK::ForbiddenStreamTarget)));
 
