@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,6 +41,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -59,14 +60,14 @@ pub mod config;
 pub mod status;
 
 pub use address::{DangerouslyIntoTorAddr, IntoTorAddr, TorAddr, TorAddrError};
-pub use builder::{TorClientBuilder, MAX_LOCAL_RESOURCE_TIMEOUT};
+pub use builder::{MAX_LOCAL_RESOURCE_TIMEOUT, TorClientBuilder};
 pub use client::{BootstrapBehavior, DormantMode, InertTorClient, StreamPrefs, TorClient};
 pub use config::TorClientConfig;
 
-pub use tor_circmgr::isolation;
 pub use tor_circmgr::IsolationToken;
+pub use tor_circmgr::isolation;
 pub use tor_error::{ErrorKind, HasKind};
-pub use tor_proto::stream::{DataReader, DataStream, DataWriter};
+pub use tor_proto::client::stream::{DataReader, DataStream, DataWriter};
 
 mod err;
 pub use err::{Error, ErrorHint, HintableError};
@@ -163,6 +164,8 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
+    use cfg_if::cfg_if;
+
     use super::*;
 
     #[test]
@@ -187,7 +190,7 @@ mod test {
             assert!(unsupported.is_empty(), "{} not supported", unsupported);
         }
 
-        // TODO: Remove this once congestion control is fully implemented.
+        // TODO: Revise this once congestion control is fully implemented and always-on.
         {
             // Recommended by c-tor main-branch authorities as of 2024-04-02
             let rec: tor_protover::Protocols =
@@ -196,12 +199,18 @@ mod test {
                     .parse()
                     .unwrap();
 
-            // Although this is recommended, Arti hasn't built it yet.
-            let expected_missing: tor_protover::Protocols =
-                [tor_protover::named::FLOWCTRL_CC].into_iter().collect();
-
+            // Although this is recommended, it isn't always-on in Arti yet yet.
+            cfg_if! {
+                if #[cfg(feature="flowctl-cc")] {
+                     let permitted_missing: tor_protover::Protocols =
+                        [].into_iter().collect();
+                } else {
+                    let permitted_missing: tor_protover::Protocols =
+                        [tor_protover::named::FLOWCTRL_CC].into_iter().collect();
+                }
+            }
             let unsupported = rec.difference(&pr);
-            assert_eq!(unsupported, expected_missing);
+            assert!(unsupported.difference(&permitted_missing).is_empty());
         }
     }
 
