@@ -20,7 +20,7 @@ use arti_client::TorClientConfig;
 #[cfg(feature = "onion-service-service")]
 use tor_config::define_list_builder_accessors;
 use tor_config::resolve_alternative_specs;
-pub(crate) use tor_config::{impl_standard_builder, ConfigBuildError, Listen};
+pub(crate) use tor_config::{ConfigBuildError, Listen, impl_standard_builder};
 
 use crate::{LoggingConfig, LoggingConfigBuilder};
 
@@ -123,6 +123,8 @@ macro_rules! resolve_listen_port {
 #[allow(clippy::option_option)] // Builder port fields: Some(None) = specified to disable
 pub struct ProxyConfig {
     /// Addresses to listen on for incoming SOCKS connections.
+    //
+    // TODO: Once http-connect is non-experimental, we should rename this option in a backward-compatible way.
     #[builder(field(build = r#"#[allow(deprecated)]
                    // We use this deprecated macro to instantiate the legacy socks_port option.
                    { resolve_listen_port!(self, socks, 9150) }
@@ -389,9 +391,9 @@ mod test {
     // Saves adding many individual #[cfg], or a sub-module
     #![cfg_attr(not(feature = "pt-client"), allow(dead_code))]
 
-    use arti_client::config::dir;
     use arti_client::config::TorClientConfigBuilder;
-    use itertools::{chain, EitherOrBoth, Itertools};
+    use arti_client::config::dir;
+    use itertools::{EitherOrBoth, Itertools, chain};
     use regex::Regex;
     use std::collections::HashSet;
     use std::fmt::Write as _;
@@ -581,6 +583,16 @@ mod test {
             ],
         );
 
+        declare_exceptions(
+            None,
+            None,
+            Recognized,
+            &[
+                // Examples exist but are not auto-testable
+                "logging.opentelemetry",
+            ],
+        );
+
         declare_exceptions(
             Some(InOld),
             Some(InNew),
@@ -904,9 +916,9 @@ example config file {which:?}, uncommented={uncommented:?}
     ///   3. Either add a trivial example for the affected key(s) (starting with just `#`)
     ///      or add the affected key(s) to `declared_config_exceptions`
     fn exhaustive_1(example_file: &str, which: WhichExample, deprecated: &[String]) {
+        use InExample::*;
         use serde_json::Value as JsValue;
         use std::collections::BTreeSet;
-        use InExample::*;
 
         let example = uncomment_example_settings(example_file);
         let example: toml::Value = toml::from_str(&example).unwrap();
@@ -1062,7 +1074,7 @@ example config file {which:?}, uncommented={uncommented:?}
         // a newly-defined config item has not been added to the list for OLDEST_SUPPORTED_CONFIG.
         assert!(
             problems.is_empty(),
- "example config {which:?} exhaustiveness check failed: {}\n-----8<-----\n{}\n-----8<-----\n",
+            "example config {which:?} exhaustiveness check failed: {}\n-----8<-----\n{}\n-----8<-----\n",
             problems.join("\n"),
             example_file,
         );
@@ -1224,7 +1236,7 @@ example config file {which:?}, uncommented={uncommented:?}
 
         #[cfg(feature = "pt-client")]
         {
-            use arti_client::config::{pt::TransportConfig, BridgesConfig};
+            use arti_client::config::{BridgesConfig, pt::TransportConfig};
             use tor_config_path::CfgPath;
 
             let bridges_got: &BridgesConfig = cfg_got.0.as_ref();
@@ -1261,37 +1273,23 @@ example config file {which:?}, uncommented={uncommented:?}
 
         let result = file.resolve_return_results::<(TorClientConfig, ArtiConfig)>();
 
-        cfg_if::cfg_if! {
-            if #[cfg(feature = "memquota")] {
-                let result = result.unwrap();
+        let result = result.unwrap();
 
-                // Test that the example config doesn't have any unrecognised keys
-                assert_eq!(result.unrecognized, []);
-                assert_eq!(result.deprecated, []);
+        // Test that the example config doesn't have any unrecognised keys
+        assert_eq!(result.unrecognized, []);
+        assert_eq!(result.deprecated, []);
 
-                let inner: &tor_memquota::testing::ConfigInner =
-                    result.value.0.system_memory().inner().unwrap();
+        let inner: &tor_memquota::testing::ConfigInner =
+            result.value.0.system_memory().inner().unwrap();
 
-                // Test that the example low_water is the default
-                // value for the example max.
-                let defaulted_low = tor_memquota::Config::builder()
-                    .max(*inner.max)
-                    .build()
-                    .unwrap();
-                let inner_defaulted_low = defaulted_low.inner().unwrap();
-                assert_eq!(inner, inner_defaulted_low);
-            } else if #[cfg(arti_features_precise)] {
-                // Test that requesting memory quota tracking generates a config error
-                // if support is compiled out.
-                let m = result.unwrap_err().report().to_string();
-                assert!(m.contains("cargo feature `memquota` disabled"), "{m:?}");
-            } else {
-                // The `tor-memquota/memquota` feature is enabled by default in tor-memquota,
-                // but the corresponding `memquota` feature is but not enabled here in `arti`.
-                // so cargo --workspace enables it in a way we can't tell.  See arti/build.rs.
-                println!("not testing memquota config, cannot figure out if it's enabled");
-            }
-        }
+        // Test that the example low_water is the default
+        // value for the example max.
+        let defaulted_low = tor_memquota::Config::builder()
+            .max(*inner.max)
+            .build()
+            .unwrap();
+        let inner_defaulted_low = defaulted_low.inner().unwrap();
+        assert_eq!(inner, inner_defaulted_low);
     }
 
     #[test]
@@ -1622,7 +1620,7 @@ example config file {which:?}, uncommented={uncommented:?}
             eprintln!("parsing\n  --\n{}\n  --", &s);
             let mut sources = tor_config::ConfigurationSources::new_empty();
             sources.push_source(
-                tor_config::ConfigurationSource::from_verbatim(s.to_string()),
+                tor_config::ConfigurationSource::from_verbatim(s.clone()),
                 tor_config::sources::MustRead::MustRead,
             );
             sources.load().expect(&s)
@@ -1646,10 +1644,9 @@ example config file {which:?}, uncommented={uncommented:?}
         use tor_config_path::CfgPath;
         let sec = std::time::Duration::from_secs(1);
 
-        let auth = dir::Authority::builder()
-            .name("Fred")
-            .v3ident([22; 20].into())
-            .clone();
+        let mut authorities = dir::AuthorityContacts::builder();
+        authorities.v3idents().push([22; 20].into());
+
         let mut fallback = dir::FallbackDir::builder();
         fallback
             .rsa_identity([23; 20].into())
@@ -1663,7 +1660,7 @@ example config file {which:?}, uncommented={uncommented:?}
         bld.proxy().socks_listen(Listen::new_localhost(9999));
         bld.logging().console("warn");
 
-        bld_tor.tor_network().set_authorities(vec![auth]);
+        *bld_tor.tor_network().authorities() = authorities;
         bld_tor.tor_network().set_fallback_caches(vec![fallback]);
         bld_tor
             .storage()
