@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -91,7 +91,7 @@ semipublic_mod! {
     mod onion_proxy;
     mod process;
     mod reload_cfg;
-    mod socks;
+    mod proxy;
 }
 #[cfg_attr(not(feature = "rpc"), path = "rpc_stub.rs")]
 mod rpc;
@@ -100,27 +100,28 @@ use std::ffi::OsString;
 use std::fmt::Write;
 
 pub use cfg::{
-    ApplicationConfig, ApplicationConfigBuilder, ArtiCombinedConfig, ArtiConfig, ArtiConfigBuilder,
-    ProxyConfig, ProxyConfigBuilder, SystemConfig, SystemConfigBuilder, ARTI_EXAMPLE_CONFIG,
+    ARTI_EXAMPLE_CONFIG, ApplicationConfig, ApplicationConfigBuilder, ArtiCombinedConfig,
+    ArtiConfig, ArtiConfigBuilder, ProxyConfig, ProxyConfigBuilder, SystemConfig,
+    SystemConfigBuilder,
 };
 pub use logging::{LoggingConfig, LoggingConfigBuilder};
 
-use arti_client::config::default_config_files;
 use arti_client::TorClient;
+use arti_client::config::default_config_files;
 use safelog::with_safe_logging_suppressed;
-use tor_config::mistrust::BuilderExt as _;
 use tor_config::ConfigurationSources;
+use tor_config::mistrust::BuilderExt as _;
 use tor_rtcompat::ToplevelRuntime;
 
 use anyhow::{Context, Error, Result};
-use clap::{value_parser, Arg, ArgAction, Command};
+use clap::{Arg, ArgAction, Command, value_parser};
 #[allow(unused_imports)]
-use tracing::{error, info, warn};
+use tracing::{error, info, instrument, warn};
 
 #[cfg(any(
     feature = "hsc",
     feature = "onion-service-service",
-    feature = "onion-service-cli-extra"
+    feature = "onion-service-cli-extra",
 ))]
 use clap::Subcommand as _;
 
@@ -135,6 +136,8 @@ fn create_runtime() -> std::io::Result<impl ToplevelRuntime> {
             use tor_rtcompat::tokio::TokioNativeTlsRuntime as ChosenRuntime;
         } else if #[cfg(all(feature="tokio", feature="rustls"))] {
             use tor_rtcompat::tokio::TokioRustlsRuntime as ChosenRuntime;
+            // Note: See comments in tor_rtcompate::impls::rustls::RustlsProvider
+            // about choice of default crypto provider.
             let _idempotent_ignore = rustls_crate::crypto::CryptoProvider::install_default(
                 rustls_crate::crypto::ring::default_provider(),
 
@@ -143,6 +146,8 @@ fn create_runtime() -> std::io::Result<impl ToplevelRuntime> {
             use tor_rtcompat::async_std::AsyncStdNativeTlsRuntime as ChosenRuntime;
         } else if #[cfg(all(feature="async-std", feature="rustls"))] {
             use tor_rtcompat::async_std::AsyncStdRustlsRuntime as ChosenRuntime;
+            // Note: See comments in tor_rtcompate::impls::rustls::RustlsProvider
+            // about choice of default crypto provider.
             let _idempotent_ignore = rustls_crate::crypto::CryptoProvider::install_default(
                 rustls_crate::crypto::ring::default_provider(),
             );
@@ -182,6 +187,7 @@ fn list_enabled_features() -> &'static [&'static str] {
 /// Currently, might panic if wrong arguments are specified.
 #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
 #[allow(clippy::cognitive_complexity)]
+#[instrument(skip_all, level = "trace")]
 fn main_main<I, T>(cli_args: I) -> Result<()>
 where
     I: IntoIterator<Item = T>,
@@ -264,9 +270,10 @@ where
             .subcommand(
                 Command::new("proxy")
                     .about(
-                        "Run Arti in SOCKS proxy mode, proxying connections through the Tor network.",
+                        "Run Arti in proxy mode, proxying connections through the Tor network.",
                     )
                     .arg(
+                        // TODO: Allow a proxy-port alias for this too, once http-connect is stable.
                         Arg::new("socks-port")
                             .short('p')
                             .action(ArgAction::Set)
@@ -302,6 +309,7 @@ where
     cfg_if::cfg_if! {
         if #[cfg(feature = "onion-service-cli-extra")] {
             let clap_app = subcommands::keys::KeysSubcommands::augment_subcommands(clap_app);
+            let clap_app = subcommands::raw::RawSubcommands::augment_subcommands(clap_app);
         }
     }
 
@@ -385,7 +393,9 @@ where
     #[cfg(feature = "harden")]
     if !config.application().permit_debugging {
         if let Err(e) = process::enable_process_hardening() {
-            error!("Encountered a problem while enabling hardening. To disable this feature, set application.permit_debugging to true.");
+            error!(
+                "Encountered a problem while enabling hardening. To disable this feature, set application.permit_debugging to true."
+            );
             return Err(e);
         }
     }
@@ -395,11 +405,13 @@ where
         return subcommands::proxy::run(runtime, proxy_matches, cfg_sources, config, client_config);
     }
 
-    // Check for the optional "keys" subcommand.
+    // Check for the optional "keys" and "keys-raw" subcommand.
     cfg_if::cfg_if! {
         if #[cfg(feature = "onion-service-cli-extra")] {
             if let Some(keys_matches) = matches.subcommand_matches("keys") {
-                return subcommands::keys::run(runtime, keys_matches, &client_config);
+                return subcommands::keys::run(runtime, keys_matches, &config, &client_config);
+            } else if let Some(raw_matches) = matches.subcommand_matches("keys-raw") {
+                return subcommands::raw::run(runtime, raw_matches, &client_config);
             }
         }
     }
@@ -408,7 +420,7 @@ where
     cfg_if::cfg_if! {
         if #[cfg(feature = "onion-service-service")] {
             if let Some(hss_matches) = matches.subcommand_matches("hss") {
-                return subcommands::hss::run(hss_matches, &config, &client_config);
+                return subcommands::hss::run(runtime, hss_matches, &config, &client_config);
             }
         }
     }
