@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // TODO: Stuff to add before this crate is ready....
 //  - Test the absolute heck out of it.
@@ -47,6 +47,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // This crate used to have unsafe code to interact with various libc functions.
@@ -60,11 +61,13 @@
 mod dir;
 mod disable;
 mod err;
+mod file_access;
 mod imp;
 #[cfg(all(
     target_family = "unix",
     not(target_os = "ios"),
-    not(target_os = "android")
+    not(target_os = "android"),
+    not(target_os = "tvos")
 ))]
 mod user;
 
@@ -84,7 +87,8 @@ use std::{
 
 pub use dir::CheckedDir;
 pub use disable::GLOBAL_DISABLE_VAR;
-pub use err::{format_access_bits, Error};
+pub use err::{Error, format_access_bits};
+pub use file_access::FileAccess;
 
 /// A result type as returned by this crate
 pub type Result<T> = std::result::Result<T, Error>;
@@ -92,7 +96,8 @@ pub type Result<T> = std::result::Result<T, Error>;
 #[cfg(all(
     target_family = "unix",
     not(target_os = "ios"),
-    not(target_os = "android")
+    not(target_os = "android"),
+    not(target_os = "tvos")
 ))]
 pub use user::{TrustedGroup, TrustedUser};
 
@@ -178,7 +183,8 @@ pub struct Mistrust {
     #[cfg(all(
         target_family = "unix",
         not(target_os = "ios"),
-        not(target_os = "android")
+        not(target_os = "android"),
+        not(target_os = "tvos")
     ))]
     #[builder(
         setter(into),
@@ -190,7 +196,8 @@ pub struct Mistrust {
     #[cfg(all(
         target_family = "unix",
         not(target_os = "ios"),
-        not(target_os = "android")
+        not(target_os = "android"),
+        not(target_os = "tvos")
     ))]
     #[builder(
         setter(into),
@@ -225,7 +232,8 @@ impl MistrustBuilder {
     #[cfg(all(
         target_family = "unix",
         not(target_os = "ios"),
-        not(target_os = "android")
+        not(target_os = "android"),
+        not(target_os = "tvos"),
     ))]
     pub fn trust_admin_only(&mut self) -> &mut Self {
         self.trust_user = TrustedUser::None;
@@ -244,7 +252,8 @@ impl MistrustBuilder {
     #[cfg(all(
         target_family = "unix",
         not(target_os = "ios"),
-        not(target_os = "android")
+        not(target_os = "android"),
+        not(target_os = "tvos"),
     ))]
     pub fn trust_no_group_id(&mut self) -> &mut Self {
         self.trust_group = TrustedGroup::None;
@@ -454,9 +463,21 @@ impl Mistrust {
     pub(crate) fn is_disabled(&self) -> bool {
         self.status.disabled()
     }
+
+    /// Create a new [`FileAccess`] for reading or writing files
+    /// while enforcing the rules of this `Mistrust`.
+    pub fn file_access(&self) -> FileAccess<'_> {
+        self.verifier().file_access()
+    }
 }
 
 impl<'a> Verifier<'a> {
+    /// Create a new [`FileAccess`] for reading or writing files
+    /// while enforcing the rules of this `Verifier`.
+    pub fn file_access(self) -> FileAccess<'a> {
+        FileAccess::from_verifier(self)
+    }
+
     /// Configure this `Verifier` to require that all paths it checks be
     /// files (not directories).
     pub fn require_file(mut self) -> Self {
@@ -535,7 +556,7 @@ impl<'a> Verifier<'a> {
 
     /// Check whether the file or directory at `path` conforms to the
     /// requirements of this `Verifier` and the [`Mistrust`] that created it.
-    pub fn check<P: AsRef<Path>>(self, path: P) -> Result<()> {
+    pub fn check<P: AsRef<Path>>(&self, path: P) -> Result<()> {
         let path = path.as_ref();
 
         // This is the powerhouse of our verifier code:
@@ -550,7 +571,7 @@ impl<'a> Verifier<'a> {
             error_iterator.collect()
         } else {
             let next = error_iterator.next();
-            drop(error_iterator); // so that "canonical" is no loner borrowed.
+            drop(error_iterator); // so that "canonical" is no longer borrowed.
             next
         };
 
@@ -573,7 +594,7 @@ impl<'a> Verifier<'a> {
     ///  * there was a problem when creating the directory
     ///  * after creating the directory, we found that it had a permissions or
     ///    ownership problem.
-    pub fn make_directory<P: AsRef<Path>>(mut self, path: P) -> Result<()> {
+    pub fn make_directory<P: AsRef<Path>>(&mut self, path: P) -> Result<()> {
         self.enforce_type = Type::Dir;
 
         let path = path.as_ref();
@@ -637,7 +658,7 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
-    use testing::{mistrust_build, Dir, MistrustOp};
+    use testing::{Dir, MistrustOp, mistrust_build};
 
     #[cfg(target_family = "unix")]
     use testing::LinkType;
