@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -55,13 +55,17 @@ mod flags;
 mod impls;
 
 pub use err::Error;
-pub use flags::{disable_safe_logging, enforce_safe_logging, with_safe_logging_suppressed, Guard};
+pub use flags::{Guard, disable_safe_logging, enforce_safe_logging, with_safe_logging_suppressed};
 
 use std::ops::Deref;
 
 /// A `Result` returned by the flag-manipulation functions in `safelog`.
 pub type Result<T> = std::result::Result<T, Error>;
 
+// Re-exported for macros.
+#[doc(hidden)]
+pub use flags::unsafe_logging_enabled;
+
 /// A wrapper type for a sensitive value.
 ///
 /// By default, a `Sensitive<T>` behaves the same as a regular `T`, except that
@@ -323,6 +327,139 @@ impl<T: Redactable> std::fmt::Debug for MaybeRedacted<T> {
     }
 }
 
+/// A type that can be displayed in a redacted or un-redacted form,
+/// but which forces the caller to choose.
+///
+/// See [`Redactable`] for more discussion on redaction.
+///
+/// Unlike [`Redactable`], this type is "inherently sensitive":
+/// Types implementing `DisplayRedacted` should not typically implement
+/// [`Display`](std::fmt::Display).
+///
+/// For external types that implement `Display`,
+/// or for types which are usually _not_ sensitive,
+/// `Redacted` is likely a better choice.
+pub trait DisplayRedacted {
+    /// As [`Display::fmt`](std::fmt::Display::fmt), but write this object
+    /// in its redacted form.
+    fn fmt_redacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result;
+    /// As [`Display::fmt`](std::fmt::Display::fmt), but write this object
+    /// in its un-redacted form.
+    fn fmt_unredacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result;
+
+    // TODO: At some point in the future, when default values are supported for GATs,
+    // it might be good to turn these RPIT functions into associated types.
+
+    /// Return a pointer wrapping this object that can be Displayed in redacted form
+    /// if safe-logging is enabled.
+    ///
+    /// (If safe-logging is not enabled, it will de displayed in its unredacted form.)
+    fn display_redacted(&self) -> impl std::fmt::Display + '_ {
+        DispRedacted(self)
+    }
+    /// Return a pointer wrapping this object that can be Displayed in unredacted form.
+    fn display_unredacted(&self) -> impl std::fmt::Display + '_ {
+        DispUnredacted(self)
+    }
+}
+
+impl<'a, T> DisplayRedacted for &'a T
+where
+    T: DisplayRedacted + ?Sized,
+{
+    fn display_redacted(&self) -> impl std::fmt::Display + '_ {
+        (*self).display_redacted()
+    }
+    fn display_unredacted(&self) -> impl std::fmt::Display + '_ {
+        (*self).display_unredacted()
+    }
+    fn fmt_redacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        (*self).fmt_redacted(f)
+    }
+    fn fmt_unredacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        (*self).fmt_unredacted(f)
+    }
+}
+
+/// A wrapper around a [`DisplayRedacted`] that implements [`Display`](std::fmt::Display)
+/// by displaying the object in its redacted form
+/// if safe-logging is enabled.
+///
+/// (If safe-logging is not enabled, it will de displayed in its unredacted form.)
+#[allow(clippy::exhaustive_structs)]
+#[derive(derive_more::AsRef)]
+pub struct DispRedacted<T: ?Sized>(pub T);
+
+/// A wrapper around a [`DisplayRedacted`] that implements [`Display`](std::fmt::Display)
+/// by displaying the object in its un-redacted form.
+#[allow(clippy::exhaustive_structs)]
+#[derive(derive_more::AsRef)]
+pub struct DispUnredacted<T: ?Sized>(pub T);
+
+impl<T> std::fmt::Display for DispRedacted<T>
+where
+    T: DisplayRedacted + ?Sized,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        if crate::flags::unsafe_logging_enabled() {
+            self.0.fmt_unredacted(f)
+        } else {
+            self.0.fmt_redacted(f)
+        }
+    }
+}
+
+impl<T> std::fmt::Display for DispUnredacted<T>
+where
+    T: DisplayRedacted + ?Sized,
+{
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        self.0.fmt_unredacted(f)
+    }
+}
+
+/// A type that can be debugged in a redacted or un-redacted form,
+/// but which forces the caller to choose.
+///
+/// See [`Redactable`] for more discussion on redaction.
+///
+/// Unlike [`Redactable`], this type is "inherently sensitive":
+/// [`Debug`](std::fmt::Debug) will display it in redacted or un-redacted format
+/// depending on whether safe logging is enabled.
+///
+/// For external types that implement `Debug`,
+/// or for types which are usually _not_ sensitive,
+/// `Redacted` is likely a better choice.
+pub trait DebugRedacted {
+    /// As [`Debug::fmt`](std::fmt::Debug::fmt), but write this object
+    /// in its redacted form.
+    fn fmt_redacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result;
+    /// As [`Debug::fmt`](std::fmt::Debug::fmt), but write this object
+    /// in its unredacted form.
+    fn fmt_unredacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result;
+}
+
+/// Implement [`std::fmt::Debug`] for a type that implements [`DebugRedacted`].
+///
+/// The implementation will use fmt_redacted() when safe-logging is enabled,
+/// and fmt_unredacted() otherwise.
+///
+/// (NOTE we can't just write 'impl<T:DebugRedacted> Debug for T`;
+/// Rust doesn't like it.)
+#[macro_export]
+macro_rules! derive_redacted_debug {
+    {$t:ty} => {
+    impl std::fmt::Debug for $t {
+        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            if $crate::unsafe_logging_enabled() {
+                $crate::DebugRedacted::fmt_unredacted(self, f)
+            } else {
+                $crate::DebugRedacted::fmt_redacted(self, f)
+            }
+        }
+    }
+}}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -338,6 +475,7 @@ mod test {
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+
     use super::*;
     use serial_test::serial;
     use static_assertions::{assert_impl_all, assert_not_impl_any};
@@ -471,4 +609,41 @@ mod test {
 
         assert_eq!(Redacted::new(localhost).unwrap(), localhost);
     }
+
+    struct RedactionCheck(u32);
+    impl DisplayRedacted for RedactionCheck {
+        fn fmt_unredacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            write!(f, "{}", self.0)
+        }
+
+        fn fmt_redacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            let v = self.0.to_string();
+            write!(f, "{}xxx", v.chars().next().unwrap())
+        }
+    }
+    impl DebugRedacted for RedactionCheck {
+        fn fmt_redacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            write!(f, "Num({})", self.display_redacted())
+        }
+
+        fn fmt_unredacted(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            write!(f, "Num({})", self.display_unredacted())
+        }
+    }
+    derive_redacted_debug!(RedactionCheck);
+
+    #[test]
+    #[serial]
+    fn display_redacted() {
+        let n = RedactionCheck(999);
+        assert_eq!(&n.display_unredacted().to_string(), "999");
+        assert_eq!(&n.display_redacted().to_string(), "9xxx");
+        with_safe_logging_suppressed(|| assert_eq!(&n.display_redacted().to_string(), "999"));
+
+        assert_eq!(DispRedacted(&n).to_string(), "9xxx");
+        assert_eq!(DispUnredacted(&n).to_string(), "999");
+
+        assert_eq!(&format!("{n:?}"), "Num(9xxx)");
+        with_safe_logging_suppressed(|| assert_eq!(&format!("{n:?}"), "Num(999)"));
+    }
 }
