@@ -3,15 +3,10 @@
 //! The memory tracker needs a way to look at the next item of a stream
 //! (if there is one, or there can immediately be one),
 //! *without* getting involved with the async tasks.
-//
-// TODO at some point this should probably be in tor-async-utils
-
-#![allow(dead_code)] // TODO #351
 
 use educe::Educe;
-use futures::stream::FusedStream;
-use futures::task::noop_waker_ref;
 use futures::Stream;
+use futures::stream::FusedStream;
 use pin_project::pin_project;
 
 use crate::peekable_stream::{PeekableStream, UnobtrusivePeekableStream};
@@ -134,7 +129,7 @@ impl<S: Stream> UnobtrusivePeekableStream for StreamUnobtrusivePeeker<S> {
             let waker = if let Some(waker) = self_.poll_waker.as_ref() {
                 waker
             } else {
-                noop_waker_ref()
+                Waker::noop()
             };
 
             match inner.poll_next(&mut Context::from_waker(waker)) {
