@@ -3,11 +3,12 @@
 //! Relay messages are sent along circuits, inside RELAY or RELAY_EARLY
 //! cells.
 
+use super::flow_ctrl::{Xoff, Xon};
 use super::{RelayCellFormat, RelayCmd};
+use crate::chancell::CELL_DATA_LEN;
 use crate::chancell::msg::{
     DestroyReason, HandshakeType, TAP_C_HANDSHAKE_LEN, TAP_S_HANDSHAKE_LEN,
 };
-use crate::chancell::CELL_DATA_LEN;
 use caret::caret_int;
 use derive_deftly::Deftly;
 use std::fmt::Write;
@@ -26,15 +27,11 @@ use bitflags::bitflags;
 #[cfg_attr(docsrs, doc(cfg(feature = "conflux")))]
 pub use super::conflux::{ConfluxLink, ConfluxLinked, ConfluxLinkedAck, ConfluxSwitch};
 
-#[cfg(feature = "flowctl-cc")]
-#[cfg_attr(docsrs, doc(cfg(feature = "flowctl-cc")))]
-use super::flow_ctrl::{Xoff, Xon};
-
 #[cfg(feature = "hs")]
 #[cfg_attr(docsrs, doc(cfg(feature = "hs")))]
 pub use super::hs::{
-    est_intro::EstablishIntro, EstablishRendezvous, IntroEstablished, Introduce1, Introduce2,
-    IntroduceAck, Rendezvous1, Rendezvous2, RendezvousEstablished,
+    EstablishRendezvous, IntroEstablished, Introduce1, Introduce2, IntroduceAck, Rendezvous1,
+    Rendezvous2, RendezvousEstablished, est_intro::EstablishIntro,
 };
 #[cfg(feature = "experimental-udp")]
 #[cfg_attr(docsrs, doc(cfg(feature = "experimental-udp")))]
@@ -99,10 +96,8 @@ pub enum AnyRelayMsg : RelayMsg {
     [feature = "conflux"]
     ConfluxSwitch,
     /// Update a stream's transmit rate limit.
-    [feature = "flowctl-cc"]
     Xon,
     /// Disable transmitting on a stream.
-    [feature = "flowctl-cc"]
     Xoff,
     /// Establish Introduction
     [feature = "hs"]
@@ -474,8 +469,8 @@ caret_int! {
 
 impl tor_error::HasKind for EndReason {
     fn kind(&self) -> tor_error::ErrorKind {
-        use tor_error::ErrorKind as EK;
         use EndReason as E;
+        use tor_error::ErrorKind as EK;
         match *self {
             E::MISC => EK::RemoteStreamError,
             E::RESOLVEFAILED => EK::RemoteHostResolutionFailed,
@@ -1434,7 +1429,6 @@ msg_impl_relaymsg!(
 #[cfg(feature = "conflux")]
 msg_impl_relaymsg!(ConfluxSwitch, ConfluxLink, ConfluxLinked, ConfluxLinkedAck);
 
-#[cfg(feature = "flowctl-cc")]
 msg_impl_relaymsg!(Xon, Xoff);
 
 #[cfg(test)]
