@@ -17,6 +17,7 @@ use tor_netdir::params::NetParameters;
 use tor_proto::channel::kist::KistParams;
 use tor_proto::channel::params::ChannelPaddingInstructionsUpdates;
 use tor_proto::memquota::{ChannelAccount, SpecificAccount as _, ToplevelAccount};
+use tracing::{instrument, trace};
 
 mod select;
 mod state;
@@ -184,6 +185,7 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
     ///
     /// If no such channel exists already, but we have one that's in
     /// progress, wait for it to succeed or fail.
+    #[instrument(skip_all, level = "trace")]
     pub(crate) async fn get_or_launch(
         &self,
         target: CF::BuildSpec,
@@ -202,6 +204,7 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
     }
 
     /// Get a channel whose identity is `ident` - internal implementation
+    #[allow(clippy::cognitive_complexity)]
     async fn get_or_launch_internal(
         &self,
         target: CF::BuildSpec,
@@ -239,10 +242,12 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
                 }
                 // Easy case: we have an error or a channel to return.
                 Some(Action::Return(v)) => {
+                    trace!("Returning existing channel");
                     return v.map(|chan| (chan, provenance));
                 }
                 // There's an in-progress channel.  Wait for it.
                 Some(Action::Wait(pend)) => {
+                    trace!("Waiting for in-progress channel");
                     match pend.await {
                         Ok(Ok(())) => {
                             // We were waiting for a channel, and it succeeded, or it
@@ -265,6 +270,7 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
                 }
                 // We need to launch a channel.
                 Some(Action::Launch((handle, send))) => {
+                    trace!("Launching channel");
                     // If the remainder of this code returns early or is cancelled, we still want to
                     // clean up our pending entry in the channel map. The following closure will be
                     // run when dropped to ensure that it's cleaned up properly.
@@ -416,7 +422,7 @@ impl<CF: AbstractChannelFactory + Clone> AbstractChanMgr<CF> {
                 channel_map
                     .by_id(ident)
                     .filter_map(|entry| match entry {
-                        Open(ref ent) if ent.channel.is_usable() => Some(Arc::clone(&ent.channel)),
+                        Open(ent) if ent.channel.is_usable() => Some(Arc::clone(&ent.channel)),
                         _ => None,
                     })
                     .collect()
@@ -457,15 +463,15 @@ mod test {
     use crate::Error;
 
     use futures::join;
-    use std::sync::atomic::{AtomicBool, Ordering};
     use std::sync::Arc;
+    use std::sync::atomic::{AtomicBool, Ordering};
     use std::time::Duration;
     use tor_error::bad_api_usage;
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_memquota::ArcMemoryQuotaTrackerExt as _;
 
     use crate::ChannelUsage as CU;
-    use tor_rtcompat::{task::yield_now, test_with_one_runtime, Runtime};
+    use tor_rtcompat::{Runtime, task::yield_now, test_with_one_runtime};
 
     #[derive(Clone)]
     struct FakeChannelFactory<RT> {
