@@ -3,25 +3,25 @@
 use std::time::Duration;
 
 use super::AbstractChannelFactory;
-use super::{select, AbstractChannel, Pending, Sending};
+use super::{AbstractChannel, Pending, Sending, select};
 use crate::{ChannelConfig, Dormancy, Error, Result};
 
 use futures::FutureExt;
 use std::result::Result as StdResult;
-use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
+use std::sync::atomic::{AtomicU64, Ordering};
 use tor_async_utils::oneshot;
 use tor_basic_utils::RngExt as _;
 use tor_cell::chancell::msg::PaddingNegotiate;
 use tor_config::PaddingLevel;
 use tor_error::{error_report, internal, into_internal};
 use tor_linkspec::{HasRelayIds, ListByRelayIds, RelayIds};
-use tor_netdir::{params::NetParameters, params::CHANNEL_PADDING_TIMEOUT_UPPER_BOUND};
+use tor_netdir::{params::CHANNEL_PADDING_TIMEOUT_UPPER_BOUND, params::NetParameters};
+use tor_proto::ChannelPaddingInstructions;
+use tor_proto::channel::ChannelPaddingInstructionsUpdates;
 use tor_proto::channel::kist::{KistMode, KistParams};
 use tor_proto::channel::padding::Parameters as PaddingParameters;
 use tor_proto::channel::padding::ParametersBuilder as PaddingParametersBuilder;
-use tor_proto::channel::ChannelPaddingInstructionsUpdates;
-use tor_proto::ChannelPaddingInstructions;
 use tor_units::{BoundedInt32, IntegerMilliseconds};
 use tracing::info;
 use void::{ResultVoidExt as _, Void};
@@ -217,7 +217,7 @@ impl From<&NetParameters> for NetParamsExtract {
         // In practice, however, the type conversion is needed
         // because consensus params are i32s.
         //
-        // See the `NetParamaters::kist_tcp_notsent_lowat docs for more details.
+        // See the `NetParameters::kist_tcp_notsent_lowat` docs for more details.
         let tcp_notsent_lowat = u32::from(p.kist_tcp_notsent_lowat);
         let kist = KistParams::new(kist_enabled, tcp_notsent_lowat);
 
@@ -523,7 +523,7 @@ impl<C: AbstractChannelFactory> MgrState<C> {
         let new_entry = ChannelState::Open(OpenEntry {
             channel,
             max_unused_duration: Duration::from_secs(
-                rand::thread_rng()
+                rand::rng()
                     .gen_range_checked(180..270)
                     .expect("not 180 < 270 !"),
             ),
