@@ -3,22 +3,22 @@
 use std::time::{Duration, Instant};
 
 use crate::{
-    hspool::{HsCircStub, HsCircStubKind},
-    AbstractCirc,
+    AbstractTunnel,
+    hspool::{HsCircStem, HsCircStemKind},
 };
 use rand::Rng;
 use tor_basic_utils::RngExt as _;
 
 /// A collection of circuits used to fulfil onion-service-related requests.
-pub(super) struct Pool<C: AbstractCirc> {
+pub(super) struct Pool<C: AbstractTunnel> {
     /// The collection of circuits themselves, in no particular order.
-    circuits: Vec<HsCircStub<C>>,
+    circuits: Vec<HsCircStem<C>>,
 
-    /// The number of SHORT elements that we would like to have in our pool.
-    stub_target: usize,
+    /// The number of NAIVE elements that we would like to have in our pool.
+    stem_target: usize,
 
-    /// The number of EXTENDED elements that we would like to have in our pool.
-    ext_stub_target: usize,
+    /// The number of GUARDED elements that we would like to have in our pool.
+    guarded_stem_target: usize,
 
     /// True if we have exhausted our pool since the last time we decided
     /// whether to change our target level.
@@ -32,27 +32,27 @@ pub(super) struct Pool<C: AbstractCirc> {
     last_changed_target: Option<Instant>,
 }
 
-/// Our default (and minimum) target SHORT pool size.
-const DEFAULT_SHORT_STUB_TARGET: usize = 3;
+/// Our default (and minimum) target NAIVE pool size.
+const DEFAULT_NAIVE_STEM_TARGET: usize = 3;
 
-/// Our default (and minimum) target EXTENDED pool size.
-const DEFAULT_EXT_STUB_TARGET: usize = 1;
+/// Our default (and minimum) target GUARDED pool size.
+const DEFAULT_GUARDED_STEM_TARGET: usize = 1;
 
-/// Our maximum target SHORT pool size.  We will never let our SHORT target grow above this
+/// Our maximum target NAIVE pool size.  We will never let our NAIVE target grow above this
 /// value.
-const MAX_SHORT_STUB_TARGET: usize = 384;
+const MAX_NAIVE_STEM_TARGET: usize = 384;
 
-/// Our maximum target EXTENDED pool size.  We will never let our EXTENDED target grow above this
+/// Our maximum target GUARDED pool size.  We will never let our GUARDED target grow above this
 /// value.
-const MAX_EXT_STUB_TARGET: usize = 128;
+const MAX_GUARDED_STEM_TARGET: usize = 128;
 
 /// A type of circuit we would like to launch.
 ///
 /// [`ForLaunch::note_circ_launched`] should be called whenever a circuit
-/// of this [`HsCircStubKind`] is launched, to decrement the internal target `count`.
+/// of this [`HsCircStemKind`] is launched, to decrement the internal target `count`.
 pub(super) struct ForLaunch<'a> {
     /// The kind of circuit we want to launch.
-    kind: HsCircStubKind,
+    kind: HsCircStemKind,
     /// How many circuits of this kind do we need?
     ///
     /// This is a mutable reference to one of the target values from [`CircsToLaunch`];
@@ -67,59 +67,59 @@ impl<'a> ForLaunch<'a> {
     }
 
     /// The kind of circuit we want to launch.
-    pub(super) fn kind(&self) -> HsCircStubKind {
+    pub(super) fn kind(&self) -> HsCircStemKind {
         self.kind
     }
 }
 
 /// The circuits we need to launch.
 pub(super) struct CircsToLaunch {
-    /// The number of SHORT circuits we want to launch.
-    stub_target: usize,
-    /// The number of EXTENDED circuits we want to launch.
-    ext_stub_target: usize,
+    /// The number of NAIVE circuits we want to launch.
+    stem_target: usize,
+    /// The number of GUARDED circuits we want to launch.
+    guarded_stem_target: usize,
 }
 
 impl CircsToLaunch {
     /// Return a [`ForLaunch`] representing a circuit we would like to launch.
     pub(super) fn for_launch(&mut self) -> ForLaunch {
-        // We start by launching SHORT circuits.
-        if self.stub_target > 0 {
+        // We start by launching NAIVE circuits.
+        if self.stem_target > 0 {
             ForLaunch {
-                kind: HsCircStubKind::Short,
-                count: &mut self.stub_target,
+                kind: HsCircStemKind::Naive,
+                count: &mut self.stem_target,
             }
         } else {
-            // If we have enough SHORT circuits, we can start launching EXTENDED ones too.
+            // If we have enough NAIVE circuits, we can start launching GUARDED ones too.
             ForLaunch {
-                kind: HsCircStubKind::Extended,
-                count: &mut self.ext_stub_target,
+                kind: HsCircStemKind::Guarded,
+                count: &mut self.guarded_stem_target,
             }
         }
     }
 
-    /// Return the number of SHORT circuits we would like to launch.
-    pub(super) fn stub(&self) -> usize {
-        self.stub_target
+    /// Return the number of NAIVE circuits we would like to launch.
+    pub(super) fn stem(&self) -> usize {
+        self.stem_target
     }
 
-    /// Return the number of EXTENDED circuits we would like to launch.
-    pub(super) fn ext_stub(&self) -> usize {
-        self.ext_stub_target
+    /// Return the number of GUARDED circuits we would like to launch.
+    pub(super) fn guarded_stem(&self) -> usize {
+        self.guarded_stem_target
     }
 
     /// Return the total number of circuits we would currently like to launch.
     pub(super) fn n_to_launch(&self) -> usize {
-        self.stub_target + self.ext_stub_target
+        self.stem_target + self.guarded_stem_target
     }
 }
 
-impl<C: AbstractCirc> Default for Pool<C> {
+impl<C: AbstractTunnel> Default for Pool<C> {
     fn default() -> Self {
         Self {
             circuits: Vec::new(),
-            stub_target: DEFAULT_SHORT_STUB_TARGET,
-            ext_stub_target: DEFAULT_EXT_STUB_TARGET,
+            stem_target: DEFAULT_NAIVE_STEM_TARGET,
+            guarded_stem_target: DEFAULT_GUARDED_STEM_TARGET,
             have_been_exhausted: false,
             have_been_under_highwater: false,
             last_changed_target: None,
@@ -127,16 +127,16 @@ impl<C: AbstractCirc> Default for Pool<C> {
     }
 }
 
-impl<C: AbstractCirc> Pool<C> {
+impl<C: AbstractTunnel> Pool<C> {
     /// Add `circ` to this pool
-    pub(super) fn insert(&mut self, circ: HsCircStub<C>) {
+    pub(super) fn insert(&mut self, circ: HsCircStem<C>) {
         self.circuits.push(circ);
     }
 
     /// Remove every circuit from this pool for which `f` returns false.
     pub(super) fn retain<F>(&mut self, f: F)
     where
-        F: FnMut(&HsCircStub<C>) -> bool,
+        F: FnMut(&HsCircStem<C>) -> bool,
     {
         self.circuits.retain(f);
     }
@@ -149,31 +149,31 @@ impl<C: AbstractCirc> Pool<C> {
     /// Return a [`CircsToLaunch`] describing the circuits we would currently like to launch.
     pub(super) fn circs_to_launch(&self) -> CircsToLaunch {
         CircsToLaunch {
-            stub_target: self.stubs_to_launch(),
-            ext_stub_target: self.ext_stubs_to_launch(),
+            stem_target: self.stems_to_launch(),
+            guarded_stem_target: self.guarded_stems_to_launch(),
         }
     }
 
-    /// Return the number of SHORT circuits we would currently like to launch.
-    fn stubs_to_launch(&self) -> usize {
+    /// Return the number of NAIVE circuits we would currently like to launch.
+    fn stems_to_launch(&self) -> usize {
         let circ_count = self
             .circuits
             .iter()
-            .filter(|c| c.kind == HsCircStubKind::Short)
+            .filter(|c| c.kind == HsCircStemKind::Naive)
             .count();
 
-        self.stub_target.saturating_sub(circ_count)
+        self.stem_target.saturating_sub(circ_count)
     }
 
-    /// Return the number of EXTENDED circuits we would currently like to launch.
-    fn ext_stubs_to_launch(&self) -> usize {
+    /// Return the number of GUARDED circuits we would currently like to launch.
+    fn guarded_stems_to_launch(&self) -> usize {
         let circ_count = self
             .circuits
             .iter()
-            .filter(|c| c.kind == HsCircStubKind::Extended)
+            .filter(|c| c.kind == HsCircStemKind::Guarded)
             .count();
 
-        self.ext_stub_target.saturating_sub(circ_count)
+        self.guarded_stem_target.saturating_sub(circ_count)
     }
 
     /// Return the total number of circuits we would like to launch.
@@ -181,7 +181,7 @@ impl<C: AbstractCirc> Pool<C> {
     /// We do not discard when we are _above_ this threshold, but we do
     /// try to build when we are low.
     fn target(&self) -> usize {
-        self.stub_target + self.ext_stub_target
+        self.stem_target + self.guarded_stem_target
     }
 
     /// If there is any circuit in this pool for which `f`  returns true and that satisfies
@@ -195,14 +195,14 @@ impl<C: AbstractCirc> Pool<C> {
         rng: &mut R,
         f: F,
         prefs: &HsCircPrefs,
-    ) -> Option<HsCircStub<C>>
+    ) -> Option<HsCircStem<C>>
     where
         R: Rng,
-        F: Fn(&HsCircStub<C>) -> bool,
+        F: Fn(&HsCircStem<C>) -> bool,
     {
-        let rv = match random_idx_where(rng, &mut self.circuits[..], |circ_stub| {
+        let rv = match random_idx_where(rng, &mut self.circuits[..], |circ_stem| {
             // First, check if any circuit matches _all_ the prefs
-            circ_stub.satisfies_prefs(prefs) && f(circ_stub)
+            circ_stem.satisfies_prefs(prefs) && f(circ_stem)
         })
         .or_else(|| {
             // Select a circuit satisfying `f` at random.
@@ -224,7 +224,7 @@ impl<C: AbstractCirc> Pool<C> {
 
     /// Update the target sizes for our pool.
     ///
-    /// This updates our target numbers of SHORT and EXTENDED circuits.
+    /// This updates our target numbers of NAIVE and GUARDED circuits.
     pub(super) fn update_target_size(&mut self, now: Instant) {
         /// Minimum amount of time that must elapse between a change and a
         /// decision to grow our pool.  We use this to control the rate of
@@ -241,34 +241,34 @@ impl<C: AbstractCirc> Pool<C> {
         let time_since_last_change = now.saturating_duration_since(*last_changed);
 
         // TODO: we may want to have separate have_been_exhausted/have_been_under_highwater
-        // flags for SHORT and EXTENDED circuits.
+        // flags for NAIVE and GUARDED circuits.
         //
-        // TODO: stub_target and ext_stub_target currently grow/shrink at the same rate,
+        // TODO: stem_target and guarded_stem_target currently grow/shrink at the same rate,
         // which is not ideal.
         //
         // Instead, we should switch to an adaptive strategy, where the two targets are updated
-        // based on how many SHORT/EXTENDED circuit requests we got.
+        // based on how many NAIVE/GUARDED circuit requests we got.
         if self.have_been_exhausted {
             if time_since_last_change < MIN_TIME_TO_GROW {
                 return;
             }
-            self.stub_target *= 2;
-            self.ext_stub_target *= 2;
+            self.stem_target *= 2;
+            self.guarded_stem_target *= 2;
         } else if !self.have_been_under_highwater {
             if time_since_last_change < MIN_TIME_TO_SHRINK {
                 return;
             }
 
-            self.stub_target /= 2;
-            self.ext_stub_target /= 2;
+            self.stem_target /= 2;
+            self.guarded_stem_target /= 2;
         }
         self.last_changed_target = Some(now);
-        self.stub_target = self
-            .stub_target
-            .clamp(DEFAULT_SHORT_STUB_TARGET, MAX_SHORT_STUB_TARGET);
-        self.ext_stub_target = self
-            .ext_stub_target
-            .clamp(DEFAULT_EXT_STUB_TARGET, MAX_EXT_STUB_TARGET);
+        self.stem_target = self
+            .stem_target
+            .clamp(DEFAULT_NAIVE_STEM_TARGET, MAX_NAIVE_STEM_TARGET);
+        self.guarded_stem_target = self
+            .guarded_stem_target
+            .clamp(DEFAULT_GUARDED_STEM_TARGET, MAX_GUARDED_STEM_TARGET);
         self.have_been_exhausted = false;
         self.have_been_under_highwater = false;
     }
@@ -288,13 +288,13 @@ impl<C: AbstractCirc> Pool<C> {
 /// Preferences for what kind of circuit to select from the pool.
 #[derive(Default, Debug, Clone)]
 pub(super) struct HsCircPrefs {
-    /// If `Some`, specifies the [`HsCircStubKind`] we would like.
-    pub(super) kind_prefs: Option<HsCircStubKind>,
+    /// If `Some`, specifies the [`HsCircStemKind`] we would like.
+    pub(super) kind_prefs: Option<HsCircStemKind>,
 }
 
 impl HsCircPrefs {
-    /// Set the preferred [`HsCircStubKind`].
-    pub(super) fn preferred_stub_kind(&mut self, kind: HsCircStubKind) -> &mut Self {
+    /// Set the preferred [`HsCircStemKind`].
+    pub(super) fn preferred_stem_kind(&mut self, kind: HsCircStemKind) -> &mut Self {
         self.kind_prefs = Some(kind);
         self
     }
