@@ -19,15 +19,16 @@ pub(crate) mod hspath;
 use std::result::Result as StdResult;
 use std::time::SystemTime;
 
+use itertools::Either;
 use rand::Rng;
 
-use tor_error::{bad_api_usage, internal, Bug};
+use tor_dircommon::fallback::FallbackDir;
+use tor_error::{Bug, bad_api_usage, internal};
 #[cfg(feature = "geoip")]
 use tor_geoip::{CountryCode, HasCountryCode};
-use tor_guardmgr::fallback::FallbackDir;
 use tor_guardmgr::{GuardMgr, GuardMonitor, GuardUsable};
 use tor_linkspec::{HasAddrs, HasRelayIds, OwnedChanTarget, OwnedCircTarget, RelayIdSet};
-use tor_netdir::{NetDir, Relay};
+use tor_netdir::{FamilyRules, NetDir, Relay};
 use tor_relay_selection::{RelayExclusion, RelaySelectionConfig, RelaySelector, RelayUsage};
 use tor_rtcompat::Runtime;
 
@@ -102,10 +103,10 @@ impl<'a> From<Relay<'a>> for MaybeOwnedRelay<'a> {
     }
 }
 impl<'a> HasAddrs for MaybeOwnedRelay<'a> {
-    fn addrs(&self) -> &[std::net::SocketAddr] {
+    fn addrs(&self) -> impl Iterator<Item = std::net::SocketAddr> {
         match self {
-            MaybeOwnedRelay::Relay(r) => r.addrs(),
-            MaybeOwnedRelay::Owned(r) => r.addrs(),
+            MaybeOwnedRelay::Relay(r) => Either::Left(r.addrs()),
+            MaybeOwnedRelay::Owned(r) => Either::Right(r.addrs()),
         }
     }
 }
@@ -264,13 +265,19 @@ impl OwnedPath {
             OwnedPath::Normal(p) => p.len(),
         }
     }
+
+    /// Return a reference to the first hop of this path, as an OwnedChanTarget.
+    pub(crate) fn first_hop_as_chantarget(&self) -> &OwnedChanTarget {
+        match self {
+            OwnedPath::ChannelOnly(ct) => ct,
+            // This access won't panic, since we enforce that path is nonempty.
+            OwnedPath::Normal(path) => path[0].chan_target(),
+        }
+    }
 }
 
 /// A path builder that builds multi-hop, anonymous paths.
-trait AnonymousPathBuilder<'a> {
-    /// Return the relay to use as exit node.
-    fn chosen_exit(&self) -> Option<&Relay<'_>>;
-
+trait AnonymousPathBuilder {
     /// Return the "target" that every chosen relay must be able to share a circuit with with.
     fn compatible_with(&self) -> Option<&OwnedChanTarget>;
 
@@ -282,8 +289,8 @@ trait AnonymousPathBuilder<'a> {
     ///
     /// Return the exit, along with the usage for a middle node corresponding
     /// to this exit.
-    fn pick_exit<'s, R: Rng>(
-        &'s self,
+    fn pick_exit<'a, R: Rng>(
+        &self,
         rng: &mut R,
         netdir: &'a NetDir,
         guard_exclusion: RelayExclusion<'a>,
@@ -293,24 +300,25 @@ trait AnonymousPathBuilder<'a> {
 
 /// Try to create and return a path corresponding to the requirements of
 /// this builder.
-fn pick_path<'a, B: AnonymousPathBuilder<'a>, R: Rng, RT: Runtime>(
+fn pick_path<'a, B: AnonymousPathBuilder, R: Rng, RT: Runtime>(
     builder: &B,
     rng: &mut R,
     netdir: DirInfo<'a>,
-    guards: Option<&GuardMgr<RT>>,
+    guards: &GuardMgr<RT>,
     config: &PathConfig,
     _now: SystemTime,
-) -> Result<(TorPath<'a>, Option<GuardMonitor>, Option<GuardUsable>)> {
+) -> Result<(TorPath<'a>, GuardMonitor, GuardUsable)> {
     let netdir = match netdir {
         DirInfo::Directory(d) => d,
         _ => {
             return Err(bad_api_usage!(
                 "Tried to build a multihop path without a network directory"
             )
-            .into())
+            .into());
         }
     };
     let rs_cfg = config.relay_selection_config();
+    let family_rules = FamilyRules::from(netdir.params());
 
     let target_exclusion = match builder.compatible_with() {
         Some(ct) => {
@@ -328,20 +336,13 @@ fn pick_path<'a, B: AnonymousPathBuilder<'a>, R: Rng, RT: Runtime>(
 
     // TODO-SPEC: Because of limitations in guard selection, we have to
     // pick the guard before the exit, which is not what our spec says.
-    let (guard, mon, usable) = select_guard(
-        rng,
-        netdir,
-        guards,
-        config,
-        builder.chosen_exit(),
-        builder.compatible_with(),
-        builder.path_kind(),
-    )?;
+    let (guard, mon, usable) = select_guard(netdir, guards, builder.compatible_with())?;
 
     let guard_exclusion = match &guard {
         MaybeOwnedRelay::Relay(r) => RelayExclusion::exclude_relays_in_same_family(
             &config.relay_selection_config(),
             vec![r.clone()],
+            family_rules,
         ),
         MaybeOwnedRelay::Owned(ct) => RelayExclusion::exclude_channel_target_family(
             &config.relay_selection_config(),
@@ -355,7 +356,7 @@ fn pick_path<'a, B: AnonymousPathBuilder<'a>, R: Rng, RT: Runtime>(
     let (exit, middle_usage) = builder.pick_exit(rng, netdir, exclusion, &rs_cfg)?;
 
     let mut family_exclusion =
-        RelayExclusion::exclude_relays_in_same_family(&rs_cfg, vec![exit.clone()]);
+        RelayExclusion::exclude_relays_in_same_family(&rs_cfg, vec![exit.clone()], family_rules);
     family_exclusion.extend(&guard_exclusion);
     let mut exclusion = family_exclusion;
     exclusion.extend(&target_exclusion);
@@ -399,100 +400,42 @@ fn ensure_unique_hops<'a>(hops: &'a [MaybeOwnedRelay<'a>]) -> StdResult<(), Bug>
 
 /// Try to select a guard corresponding to the requirements of
 /// this builder.
-fn select_guard<'a, R: Rng, RT: Runtime>(
-    rng: &mut R,
+fn select_guard<'a, RT: Runtime>(
     netdir: &'a NetDir,
-    guards: Option<&GuardMgr<RT>>,
-    config: &PathConfig,
-    chosen_exit: Option<&Relay<'_>>,
+    guardmgr: &GuardMgr<RT>,
     compatible_with: Option<&OwnedChanTarget>,
-    path_kind: &'static str,
-) -> Result<(
-    MaybeOwnedRelay<'a>,
-    Option<GuardMonitor>,
-    Option<GuardUsable>,
-)> {
-    let path_is_fully_random = chosen_exit.is_none();
-    match guards {
-        Some(guardmgr) => {
-            // TODO: Extract this section into its own function, and see
-            // what it can share with tor_relay_selection.
-            let mut b = tor_guardmgr::GuardUsageBuilder::default();
-            b.kind(tor_guardmgr::GuardUsageKind::Data);
-            if let Some(exit_relay) = chosen_exit {
-                // TODO(nickm): Our way of building a family here is
-                // somewhat questionable. We're only adding the ed25519
-                // identities of the exit relay and its family to the
-                // RelayId set.  That's fine for now, since we will only use
-                // relays at this point if they have a known Ed25519
-                // identity.  But if in the future the ed25519 identity
-                // becomes optional, this will need to change.
-                let mut family = RelayIdSet::new();
-                family.insert(*exit_relay.id());
-                // TODO(nickm): See "limitations" note on `known_family_members`.
-                family.extend(netdir.known_family_members(exit_relay).map(|r| *r.id()));
-                b.restrictions()
-                    .push(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
-            }
-            if let Some(avoid_target) = compatible_with {
-                let mut family = RelayIdSet::new();
-                family.extend(avoid_target.identities().map(|id| id.to_owned()));
-                if let Some(avoid_relay) = netdir.by_ids(avoid_target) {
-                    family.extend(netdir.known_family_members(&avoid_relay).map(|r| *r.id()));
-                }
-                b.restrictions()
-                    .push(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
-            }
-            let guard_usage = b.build().expect("Failed while building guard usage!");
-            let (guard, mut mon, usable) = guardmgr.select_guard(guard_usage)?;
-            let guard = if let Some(ct) = guard.as_circ_target() {
-                // This is a bridge; we will not look for it in the network directory.
-                MaybeOwnedRelay::from(ct.clone())
-            } else {
-                // Look this up in the network directory: we expect to find a relay.
-                guard
-                    .get_relay(netdir)
-                    .ok_or_else(|| {
-                        internal!(
-                            "Somehow the guardmgr gave us an unlisted guard {:?}!",
-                            guard
-                        )
-                    })?
-                    .into()
-            };
-            if !path_is_fully_random {
-                // We were given a specific exit relay to use, and
-                // the choice of exit relay might be forced by
-                // something outside of our control.
-                //
-                // Therefore, we must not blame the guard for any failure
-                // to complete the circuit.
-                mon.ignore_indeterminate_status();
-            }
-            Ok((guard, Some(mon), Some(usable)))
-        }
-        None => {
-            // TODO: compatible_with is not used if the guardmgr is `None`.
-            // This means we allow the first hop to be the same as the circuit target,
-            // which seems like a bug.
-            // (OTOH, we have decided to remove support for disabling guards,
-            // see #1465).
-            let rs_cfg = config.relay_selection_config();
-            let exclusion = match chosen_exit {
-                Some(r) => RelayExclusion::exclude_relays_in_same_family(&rs_cfg, vec![r.clone()]),
-                None => RelayExclusion::no_relays_excluded(),
-            };
-            let selector = RelaySelector::new(RelayUsage::new_guard(), exclusion);
-            let (relay, info) = selector.select_relay(rng, netdir);
-            let relay = relay.ok_or_else(|| Error::NoRelay {
-                path_kind,
-                role: "entry",
-                problem: info.to_string(),
-            })?;
-
-            Ok((MaybeOwnedRelay::from(relay), None, None))
+) -> Result<(MaybeOwnedRelay<'a>, GuardMonitor, GuardUsable)> {
+    // TODO: Extract this section into its own function, and see
+    // what it can share with tor_relay_selection.
+    let mut b = tor_guardmgr::GuardUsageBuilder::default();
+    b.kind(tor_guardmgr::GuardUsageKind::Data);
+    if let Some(avoid_target) = compatible_with {
+        let mut family = RelayIdSet::new();
+        family.extend(avoid_target.identities().map(|id| id.to_owned()));
+        if let Some(avoid_relay) = netdir.by_ids(avoid_target) {
+            family.extend(netdir.known_family_members(&avoid_relay).map(|r| *r.id()));
         }
+        b.restrictions()
+            .push(tor_guardmgr::GuardRestriction::AvoidAllIds(family));
     }
+    let guard_usage = b.build().expect("Failed while building guard usage!");
+    let (guard, mon, usable) = guardmgr.select_guard(guard_usage)?;
+    let guard = if let Some(ct) = guard.as_circ_target() {
+        // This is a bridge; we will not look for it in the network directory.
+        MaybeOwnedRelay::from(ct.clone())
+    } else {
+        // Look this up in the network directory: we expect to find a relay.
+        guard
+            .get_relay(netdir)
+            .ok_or_else(|| {
+                internal!(
+                    "Somehow the guardmgr gave us an unlisted guard {:?}!",
+                    guard
+                )
+            })?
+            .into()
+    };
+    Ok((guard, mon, usable))
 }
 
 /// For testing: make sure that `path` is the same when it is an owned
