@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,6 +41,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 pub mod cmdline;
@@ -105,7 +106,7 @@ impl ConfigurationTree {
         use figment::value::Value as V;
         let val = self.0.find_value(key).map_err(ConfigError::from_cfg_err)?;
         Ok(match val {
-            V::String(_, s) => s.to_string(),
+            V::String(_, s) => s.clone(),
             V::Num(_, n) => n.to_i128().expect("Failed to extract i128").to_string(),
             _ => format!("{:?}", val),
         })
