@@ -182,8 +182,6 @@ impl ConfigurationSources {
     // The other inputs are always used and therefore
     // don't need to be lifted into FnOnce() -> Result.
     ///
-    /// `mistrust` is used to check whether the configuration files have appropriate permissions.
-    ///
     /// `ConfigurationSource::Dir`s
     /// will be scanned for files whose name ends in `.toml`.
     /// All those files (if any) will be read (in lexical order by filename).
@@ -238,12 +236,31 @@ impl ConfigurationSources {
         self.options.push(option.into());
     }
 
+    /// All given override options.
+    ///
+    /// See [`push_option`](Self::push_option).
+    pub fn options(&self) -> impl Iterator<Item = &String> + Clone {
+        self.options.iter()
+    }
+
     /// Sets the filesystem permission mistrust
+    ///
+    /// This value only indicates whether and how to check permissions
+    /// on the configuration file itself.
+    /// It *does not* specify whether and how to check permissions of the
+    /// paths provided within.
+    /// This is defined by the `storage.permissions.dangerously_trust_everyone` flag.
     pub fn set_mistrust(&mut self, mistrust: fs_mistrust::Mistrust) {
         self.mistrust = mistrust;
     }
 
     /// Reads the filesystem permission mistrust
+    ///
+    /// This value only indicates whether and how to check permissions
+    /// on the configuration file itself.
+    /// It *does not* specify whether and how to check permissions of the
+    /// paths provided within.
+    /// This is defined by the `storage.permissions.dangerously_trust_everyone` flag.
     pub fn mistrust(&self) -> &fs_mistrust::Mistrust {
         &self.mistrust
     }
@@ -251,7 +268,7 @@ impl ConfigurationSources {
     /// Scan for files and load the configuration into a new [`ConfigurationTree`].
     ///
     /// This is a convenience method for [`scan()`](Self::scan)
-    /// followed by [`files.load`].
+    /// followed by `files.load`.
     pub fn load(&self) -> Result<ConfigurationTree, ConfigError> {
         let files = self.scan()?;
         files.load()
