@@ -73,19 +73,22 @@
 //! We can also transition from `Broken`, `DegradedReachable`, or `DegradedUnreachable`
 //! back to `Bootstrapping` (those transitions were omitted for brevity).
 
+use tor_circmgr::ServiceOnionServiceDirTunnel;
 use tor_config::file_watcher::{
     self, Event as FileEvent, FileEventReceiver, FileEventSender, FileWatcher, FileWatcherBuilder,
 };
 use tor_config_path::{CfgPath, CfgPathResolver};
+use tor_dirclient::SourceInfo;
 use tor_netdir::{DirEvent, NetDir};
 
+use crate::config::OnionServiceConfigPublisherView;
 use crate::config::restricted_discovery::{
     DirectoryKeyProviderList, RestrictedDiscoveryConfig, RestrictedDiscoveryKeys,
 };
-use crate::config::OnionServiceConfigPublisherView;
 use crate::status::{DescUploadRetryError, Problem};
 
 use super::*;
+use derive_more::From;
 
 // TODO-CLIENT-AUTH: perhaps we should add a separate CONFIG_CHANGE_REPUBLISH_DEBOUNCE_INTERVAL
 // for rate-limiting the publish jobs triggered by a change in the config?
@@ -240,7 +243,11 @@ impl<R: Runtime, M: Mockable> Immutable<R, M> {
                     // run out of its pre-previsioned keys).
                     //
                     // This will be addressed when we add support for offline hs_id mode
-                    .ok_or_else(|| internal!("identity keys are offline, but descriptor signing key is unavailable?!"))?
+                    .ok_or_else(|| {
+                        internal!(
+                            "identity keys are offline, but descriptor signing key is unavailable?!"
+                        )
+                    })?
                     .into();
                 key.to_bytes()
             }
@@ -288,18 +295,17 @@ pub(crate) trait Mockable: Clone + Send + Sync + Sized + 'static {
     type Rng: rand::Rng + rand::CryptoRng;
 
     /// The type of client circuit.
-    type ClientCirc: MockableClientCirc;
+    type Tunnel: MockableDirTunnel;
 
     /// Return a random number generator.
     fn thread_rng(&self) -> Self::Rng;
 
     /// Create a circuit of the specified `kind` to `target`.
-    async fn get_or_launch_specific<T>(
+    async fn get_or_launch_hs_dir<T>(
         &self,
         netdir: &NetDir,
-        kind: HsCircKind,
         target: T,
-    ) -> Result<Arc<Self::ClientCirc>, tor_circmgr::Error>
+    ) -> Result<Self::Tunnel, tor_circmgr::Error>
     where
         T: CircTarget + Send + Sync;
 
@@ -313,21 +319,28 @@ pub(crate) trait Mockable: Clone + Send + Sync + Sized + 'static {
 
 /// Mockable client circuit
 #[async_trait]
-pub(crate) trait MockableClientCirc: Send + Sync {
+pub(crate) trait MockableDirTunnel: Send + Sync {
     /// The data stream type.
     type DataStream: AsyncRead + AsyncWrite + Send + Unpin;
 
     /// Start a new stream to the last relay in the circuit, using
     /// a BEGIN_DIR cell.
-    async fn begin_dir_stream(self: Arc<Self>) -> Result<Self::DataStream, tor_proto::Error>;
+    async fn begin_dir_stream(&self) -> Result<Self::DataStream, tor_circmgr::Error>;
+
+    /// Try to get a SourceInfo for this circuit, for using it in a directory request.
+    fn source_info(&self) -> tor_proto::Result<Option<SourceInfo>>;
 }
 
 #[async_trait]
-impl MockableClientCirc for ClientCirc {
-    type DataStream = tor_proto::stream::DataStream;
+impl MockableDirTunnel for ServiceOnionServiceDirTunnel {
+    type DataStream = tor_proto::client::stream::DataStream;
 
-    async fn begin_dir_stream(self: Arc<Self>) -> Result<Self::DataStream, tor_proto::Error> {
-        ClientCirc::begin_dir_stream(self).await
+    async fn begin_dir_stream(&self) -> Result<Self::DataStream, tor_circmgr::Error> {
+        Self::begin_dir_stream(self).await
+    }
+
+    fn source_info(&self) -> tor_proto::Result<Option<SourceInfo>> {
+        SourceInfo::from_tunnel(self)
     }
 }
 
@@ -338,22 +351,21 @@ pub(crate) struct Real<R: Runtime>(Arc<HsCircPool<R>>);
 #[async_trait]
 impl<R: Runtime> Mockable for Real<R> {
     type Rng = rand::rngs::ThreadRng;
-    type ClientCirc = ClientCirc;
+    type Tunnel = ServiceOnionServiceDirTunnel;
 
     fn thread_rng(&self) -> Self::Rng {
         rand::rng()
     }
 
-    async fn get_or_launch_specific<T>(
+    async fn get_or_launch_hs_dir<T>(
         &self,
         netdir: &NetDir,
-        kind: HsCircKind,
         target: T,
-    ) -> Result<Arc<ClientCirc>, tor_circmgr::Error>
+    ) -> Result<Self::Tunnel, tor_circmgr::Error>
     where
         T: CircTarget + Send + Sync,
     {
-        self.0.get_or_launch_specific(netdir, kind, target).await
+        self.0.get_or_launch_svc_dir(netdir, target).await
     }
 
     fn estimate_upload_timeout(&self) -> Duration {
@@ -534,7 +546,7 @@ pub enum UploadError {
 
     /// Failed to establish stream to hidden service directory
     #[error("failed to establish directory stream to HsDir")]
-    Stream(#[source] tor_proto::Error),
+    Stream(#[source] tor_circmgr::Error),
 
     /// An internal error.
     #[error("Internal error")]
@@ -542,6 +554,19 @@ pub enum UploadError {
 }
 define_asref_dyn_std_error!(UploadError);
 
+impl UploadError {
+    /// Return true if this error is one that we should report as a suspicious event,
+    /// along with the dirserver, and description of the relevant document.
+    pub(crate) fn should_report_as_suspicious(&self) -> bool {
+        match self {
+            UploadError::Request(e) => e.error.should_report_as_suspicious_if_anon(),
+            UploadError::Circuit(_) => false, // TODO prop360
+            UploadError::Stream(_) => false,  // TODO prop360
+            UploadError::Bug(_) => false,
+        }
+    }
+}
+
 impl<R: Runtime, M: Mockable> Reactor<R, M> {
     /// Create a new `Reactor`.
     #[allow(clippy::too_many_arguments)]
@@ -1474,6 +1499,12 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
             Fatal(#[from] FatalError),
         }
 
+        let max_hsdesc_len: usize = netdir
+            .params()
+            .hsdir_max_desc_size
+            .try_into()
+            .expect("Unable to convert positive int32 to usize!?");
+
         let upload_results = futures::stream::iter(hs_dirs)
             .map(|relay_ids| {
                 let netdir = netdir.clone();
@@ -1572,6 +1603,7 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
                                 &mut rng,
                                 &mut key_rng,
                                 imm.runtime.wallclock(),
+                                max_hsdesc_len,
                             )?
                         };
 
@@ -1703,21 +1735,20 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
             "starting descriptor upload",
         );
 
-        let circuit = imm
+        let tunnel = imm
             .mockable
-            .get_or_launch_specific(
-                netdir,
-                HsCircKind::SvcHsDir,
-                OwnedCircTarget::from_circ_target(hsdir),
-            )
+            .get_or_launch_hs_dir(netdir, OwnedCircTarget::from_circ_target(hsdir))
             .await?;
+        let source: Option<SourceInfo> = tunnel
+            .source_info()
+            .map_err(into_internal!("Couldn't get SourceInfo for circuit"))?;
 
-        let mut stream = circuit
+        let mut stream = tunnel
             .begin_dir_stream()
             .await
             .map_err(UploadError::Stream)?;
 
-        let _response: String = send_request(&imm.runtime, &request, &mut stream, None)
+        let _response: String = send_request(&imm.runtime, &request, &mut stream, source)
             .await
             .map_err(|dir_error| -> UploadError {
                 match dir_error {
@@ -1765,8 +1796,25 @@ impl<R: Runtime, M: Mockable> Reactor<R, M> {
             imm.runtime.clone(),
         );
 
-        let fallible_op =
-            || Self::upload_descriptor(hsdesc.clone(), netdir, hsdir, Arc::clone(&imm));
+        let fallible_op = || async {
+            let r = Self::upload_descriptor(hsdesc.clone(), netdir, hsdir, Arc::clone(&imm)).await;
+
+            if let Err(e) = &r {
+                if e.should_report_as_suspicious() {
+                    // Note that not every protocol violation is suspicious:
+                    // we only warn on the protocol violations that look like attempts
+                    // to do a traffic tagging attack via hsdir inflation.
+                    // (See proposal 360.)
+                    warn_report!(
+                        e,
+                        "Suspicious error while uploading descriptor to {}/{}",
+                        ed_id,
+                        rsa_id
+                    );
+                }
+            }
+            r
+        };
 
         let outcome: Result<(), BackoffError<UploadError>> = runner.run(fallible_op).await;
         match outcome {
