@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,6 +41,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 #[cfg(feature = "decode")]
@@ -55,9 +56,9 @@ mod transport;
 pub mod verbatim;
 
 pub use ids::{
+    RelayId, RelayIdError, RelayIdRef, RelayIdType, RelayIdTypeIter,
     by_id::{ByRelayIds, ByRelayIdsError, ListByRelayIds, ListByRelayIdsError, ListByRelayIdsIter},
     set::RelayIdSet,
-    RelayId, RelayIdError, RelayIdRef, RelayIdType, RelayIdTypeIter,
 };
 pub use ls::{EncodedLinkSpec, LinkSpec, LinkSpecType};
 pub use owned::{
