@@ -22,26 +22,31 @@ pub(crate) use std::{
     fmt::{self, Debug},
     future::Future,
     marker::PhantomData,
-    mem,
-    panic::{catch_unwind, AssertUnwindSafe},
+    mem::{self, size_of},
+    panic::{AssertUnwindSafe, catch_unwind},
     pin::Pin,
     sync::{Arc, Mutex, MutexGuard, PoisonError, Weak},
 };
 
 pub(crate) use futures::{
+    FutureExt as _, Sink, SinkExt as _, Stream, StreamExt as _,
     channel::mpsc,
     stream::FusedStream,
-    task::{noop_waker_ref, Spawn, SpawnError, SpawnExt as _},
-    FutureExt as _, Sink, SinkExt as _, Stream, StreamExt as _,
+    task::{Spawn, SpawnError, SpawnExt as _},
 };
 
+pub(crate) use std::task::Waker;
+
 pub(crate) use {
-    derive_deftly::{define_derive_deftly, Deftly},
+    derive_deftly::{Deftly, define_derive_deftly, derive_deftly_adhoc},
     derive_more::{Constructor, Deref, DerefMut},
+    dyn_clone::DynClone,
     educe::Educe,
+    itertools::chain,
+    paste::paste,
     pin_project::pin_project,
     serde::{Deserialize, Serialize},
-    slotmap::SlotMap,
+    slotmap_careful::SlotMap,
     static_assertions::assert_not_impl_any,
     thiserror::Error,
     tracing::{debug, error, info},
@@ -49,12 +54,13 @@ pub(crate) use {
 };
 
 pub(crate) use {
+    tor_async_utils::mpsc_channel_no_memquota,
     tor_async_utils::stream_peek::StreamUnobtrusivePeeker,
     tor_basic_utils::ByteQty as Qty,
-    tor_config::{ConfigBuildError, ReconfigureError},
-    tor_error::{error_report, internal, into_internal, Bug, ErrorKind, HasKind},
+    tor_config::{ConfigBuildError, ExplicitOrAuto, ReconfigureError},
+    tor_error::{Bug, ErrorKind, HasKind, error_report, internal, into_internal, trace_report},
     tor_log_ratelim::log_ratelim,
-    tor_rtcompat::{CoarseInstant, CoarseTimeProvider},
+    tor_rtcompat::{CoarseInstant, CoarseTimeProvider, DynTimeProvider},
 };
 
 pub(crate) use crate::{
@@ -64,7 +70,7 @@ pub(crate) use crate::{
     error::{Error, ReclaimCrashed, StartupError, TrackerCorrupted},
     if_enabled::{EnabledToken, IfEnabled},
     memory_cost::{HasMemoryCost, HasTypedMemoryCost, TypedParticipation},
-    mtracker::{self, Account, IsParticipant, Participation},
+    mtracker::{self, Account, IsParticipant, MemoryQuotaTracker, Participation},
     private::Sealed,
     refcount,
     utils::DefaultExtTake,
