@@ -54,21 +54,29 @@ pub mod vote;
 #[cfg(feature = "build_docs")]
 mod build;
 
+#[cfg(feature = "parse2")]
+use {
+    crate::parse2::{self, ArgumentStream}, //
+};
+
 use crate::doc::authcert::{AuthCert, AuthCertKeyIds};
 use crate::parse::keyword::Keyword;
 use crate::parse::parser::{Section, SectionRules, SectionRulesBuilder};
 use crate::parse::tokenize::{Item, ItemResult, NetDocReader};
 use crate::types::misc::*;
 use crate::util::PeekableIterator;
-use crate::{Error, NetdocErrorKind as EK, Pos, Result};
-use std::collections::{HashMap, HashSet};
+use crate::{Error, NetdocErrorKind as EK, NormalItemArgument, Pos, Result};
+use std::collections::{BTreeSet, HashMap, HashSet};
+use std::fmt::{self, Display};
 use std::result::Result as StdResult;
+use std::str::FromStr;
 use std::sync::Arc;
 use std::{net, result, time};
 use tor_error::{HasKind, internal};
 use tor_protover::Protocols;
 
 use bitflags::bitflags;
+use derive_deftly::{Deftly, define_derive_deftly};
 use digest::Digest;
 use std::sync::LazyLock;
 use tor_checkable::{ExternallySigned, timed::TimerangeBound};
@@ -87,11 +95,7 @@ ns_export_each_flavor! {
 }
 
 ns_export_each_variety! {
-    ty: RouterStatus;
-}
-#[cfg(feature = "dangerous-expose-struct-fields")]
-ns_export_each_variety! {
-    ty: Header;
+    ty: RouterStatus, Preamble;
 }
 
 use void::ResultVoidExt as _;
@@ -109,79 +113,161 @@ pub use UncheckedPlainConsensus as UncheckedNsConsensus;
 #[cfg(feature = "ns_consensus")]
 pub use UnvalidatedPlainConsensus as UnvalidatedNsConsensus;
 
+#[cfg(feature = "ns-vote")]
+pub use rs::RouterStatusMdDigestsVote;
+
+/// `publiscation` field in routerstatus entry intro item other than in votes
+///
+/// Two arguments which are both ignored.
+/// This used to be an ISO8601 timestamp in anomalous two-argument format.
+///
+/// Nowadays, according to the spec, it can be a dummy value.
+/// So it can be a unit type.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:r>,
+/// except in votes which use [`Iso8601TimeSp`] instead.
+///
+/// **Not the same as** the `published` item:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Ord, PartialOrd, Default)]
+#[allow(clippy::exhaustive_structs)]
+pub struct IgnoredPublicationTimeSp;
+
 /// The lifetime of a networkstatus document.
 ///
 /// In a consensus, this type describes when the consensus may safely
 /// be used.  In a vote, this type describes the proposed lifetime for a
 /// consensus.
-#[derive(Clone, Debug)]
+///
+/// Aggregate of three netdoc preamble fields.
+#[derive(Clone, Debug, Deftly)]
+#[derive_deftly(Lifetime)]
+#[cfg_attr(feature = "parse2", derive_deftly(NetdocParseableFields))]
 pub struct Lifetime {
-    /// Time at which the document becomes valid
-    valid_after: time::SystemTime,
-    /// Time after which there is expected to be a better version
-    /// of this consensus
-    fresh_until: time::SystemTime,
-    /// Time after which this consensus is expired.
+    /// `valid-after` --- Time at which the document becomes valid
     ///
-    /// (In practice, Tor clients will keep using documents for a while
-    /// after this expiration time, if no better one can be found.)
-    valid_until: time::SystemTime,
-}
-
-impl Lifetime {
-    /// Construct a new Lifetime.
-    pub fn new(
-        valid_after: time::SystemTime,
-        fresh_until: time::SystemTime,
-        valid_until: time::SystemTime,
-    ) -> Result<Self> {
-        if valid_after < fresh_until && fresh_until < valid_until {
-            Ok(Lifetime {
-                valid_after,
-                fresh_until,
-                valid_until,
-            })
-        } else {
-            Err(EK::InvalidLifetime.err())
-        }
-    }
-    /// Return time when this consensus first becomes valid.
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// (You might see a consensus a little while before this time,
     /// since voting tries to finish up before the.)
-    pub fn valid_after(&self) -> time::SystemTime {
-        self.valid_after
-    }
-    /// Return time when this consensus is no longer fresh.
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    valid_after: Iso8601TimeSp,
+    /// `fresh-until` --- Time after which there is expected to be a better version
+    /// of this consensus
+    ///
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// You can use the consensus after this time, but there is (or is
     /// supposed to be) a better one by this point.
-    pub fn fresh_until(&self) -> time::SystemTime {
-        self.fresh_until
-    }
-    /// Return the time when this consensus is no longer valid.
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    fresh_until: Iso8601TimeSp,
+    /// `valid-until` --- Time after which this consensus is expired.
+    ///
+    /// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:published>
     ///
     /// You should try to get a better consensus after this time,
     /// though it's okay to keep using this one if no more recent one
     /// can be found.
-    pub fn valid_until(&self) -> time::SystemTime {
-        self.valid_until
-    }
-    /// Return true if this consensus is officially valid at the provided time.
-    pub fn valid_at(&self, when: time::SystemTime) -> bool {
-        self.valid_after <= when && when <= self.valid_until
-    }
+    #[cfg_attr(feature = "parse2", deftly(netdoc(single_arg)))]
+    valid_until: Iso8601TimeSp,
+}
 
-    /// Return the voting period implied by this lifetime.
-    ///
-    /// (The "voting period" is the amount of time in between when a consensus first
-    /// becomes valid, and when the next consensus is expected to become valid)
-    pub fn voting_period(&self) -> time::Duration {
-        let valid_after = self.valid_after();
-        let fresh_until = self.fresh_until();
-        fresh_until
-            .duration_since(valid_after)
-            .expect("Mis-formed lifetime")
+define_derive_deftly! {
+    /// Bespoke derive for `Lifetime`, for `new` and accessors
+    Lifetime:
+
+    impl Lifetime {
+        /// Construct a new Lifetime.
+        pub fn new(
+            $( $fname: time::SystemTime, )
+        ) -> Result<Self> {
+            // Make this now because otherwise literal `valid_after` here in the body
+            // has the wrong span - the compiler refuses to look at the argument.
+            // But we can refer to the field names.
+            let self_ = Lifetime {
+                $( $fname: $fname.into(), )
+            };
+            if self_.valid_after < self_.fresh_until && self_.fresh_until < self_.valid_until {
+                Ok(self_)
+            } else {
+                Err(EK::InvalidLifetime.err())
+            }
+        }
+      $(
+        ${fattrs doc}
+        pub fn $fname(&self) -> time::SystemTime {
+            *self.$fname
+        }
+      )
+        /// Return true if this consensus is officially valid at the provided time.
+        pub fn valid_at(&self, when: time::SystemTime) -> bool {
+            *self.valid_after <= when && when <= *self.valid_until
+        }
+
+        /// Return the voting period implied by this lifetime.
+        ///
+        /// (The "voting period" is the amount of time in between when a consensus first
+        /// becomes valid, and when the next consensus is expected to become valid)
+        pub fn voting_period(&self) -> time::Duration {
+            let valid_after = self.valid_after();
+            let fresh_until = self.fresh_until();
+            fresh_until
+                .duration_since(valid_after)
+                .expect("Mis-formed lifetime")
+        }
+    }
+}
+use derive_deftly_template_Lifetime;
+
+/// A single consensus method
+///
+/// These are integers, but we don't do arithmetic on them.
+///
+/// As defined here:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-methods>
+/// <https://spec.torproject.org/dir-spec/computing-consensus.html#flavor:microdesc>
+///
+/// As used in a `consensus-method` item:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-method>
+#[derive(Debug, Clone, Default, Eq, PartialEq, Ord, PartialOrd, Hash, Copy)] //
+#[derive(derive_more::From, derive_more::Into, derive_more::Display, derive_more::FromStr)]
+pub struct ConsensusMethod(u32);
+impl NormalItemArgument for ConsensusMethod {}
+
+/// A set of consensus methods
+///
+/// Implements `ItemValueParseable` as required for `consensus-methods`,
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:consensus-methods>
+///
+/// There is also [`consensus_methods_comma_separated`] for `m` lines in votes.
+#[derive(Debug, Clone, Default, Eq, PartialEq)]
+#[cfg_attr(feature = "parse2", derive(Deftly), derive_deftly(ItemValueParseable))]
+#[non_exhaustive]
+pub struct ConsensusMethods {
+    /// Consensus methods.
+    pub methods: BTreeSet<ConsensusMethod>,
+}
+
+/// Module for use with parse2's `with`, to parse one argument of comma-separated consensus methods
+///
+/// As found in an `m` item in a vote:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:m>
+#[cfg(feature = "parse2")]
+pub mod consensus_methods_comma_separated {
+    use super::*;
+    use parse2::ArgumentError as AE;
+    use std::result::Result;
+
+    /// Parse
+    pub fn from_args<'s>(args: &mut ArgumentStream<'s>) -> Result<ConsensusMethods, AE> {
+        let mut methods = BTreeSet::new();
+        for ent in args.next().ok_or(AE::Missing)?.split(',') {
+            let ent = ent.parse().map_err(|_| AE::Invalid)?;
+            if !methods.insert(ent) {
+                return Err(AE::Invalid);
+            }
+        }
+        Ok(ConsensusMethods { methods })
     }
 }
 
@@ -193,6 +279,13 @@ impl Lifetime {
 ///
 /// A `NetParams<i32>` is part of the validated directory manager configuration,
 /// where it is built (in the builder-pattern sense) from a transparent HashMap.
+///
+/// As found in `params` in a network status:
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:params>
+///
+/// The same syntax is also used, and this type used for parsing, in various other places,
+/// for example routerstatus entry `w` items (bandwith weights):
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:w>
 #[derive(Debug, Clone, Default, Eq, PartialEq)]
 pub struct NetParams<T> {
     /// Map from keys to values.
@@ -250,14 +343,23 @@ where
 
 /// A list of subprotocol versions that implementors should/must provide.
 ///
+/// This struct represents a pair of (optional) items:
+/// `recommended-FOO-protocols` and `required-FOO-protocols`.
+///
 /// Each consensus has two of these: one for relays, and one for clients.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:required-relay-protocols>
 #[derive(Debug, Clone, Default, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct ProtoStatus {
     /// Set of protocols that are recommended; if we're missing a protocol
     /// in this list we should warn the user.
+    ///
+    /// `recommended-client-protocols` or `recommended-relay-protocols`
     recommended: Protocols,
     /// Set of protocols that are required; if we're missing a protocol
     /// in this list we should refuse to start.
+    ///
+    /// `required-client-protocols` or `required-relay-protocols`
     required: Protocols,
 }
 
@@ -322,6 +424,10 @@ impl HasKind for ProtocolSupportError {
 
 /// A set of recommended and required protocols when running
 /// in various scenarios.
+///
+/// Represents the collection of four items: `{recommended,required}-{client,relay}-protocols`.
+///
+/// <https://spec.torproject.org/dir-spec/consensus-formats.html#item:required-relay-protocols>
 #[derive(Clone, Debug, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct ProtoStatuses {
     /// Lists of recommended and required subprotocol versions for clients
@@ -343,6 +449,8 @@ impl ProtoStatuses {
 }
 
 /// A recognized 'flavor' of consensus document.
+///
+/// <https://spec.torproject.org/dir-spec/computing-consensus.html#flavors>
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, Ord, PartialOrd)]
 #[non_exhaustive]
 pub enum ConsensusFlavor {
@@ -380,47 +488,31 @@ impl ConsensusFlavor {
 }
 
 /// The signature of a single directory authority on a networkstatus document.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub struct Signature {
     /// The name of the digest algorithm used to make the signature.
     ///
     /// Currently sha1 and sh256 are recognized.  Here we only support
     /// sha256.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    digestname: String,
+    pub digestname: String,
     /// Fingerprints of the keys for the authority that made
     /// this signature.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    key_ids: AuthCertKeyIds,
+    pub key_ids: AuthCertKeyIds,
     /// The signature itself.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    signature: Vec<u8>,
+    pub signature: Vec<u8>,
 }
 
 /// A collection of signatures that can be checked on a networkstatus document
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub struct SignatureGroup {
     /// The sha256 of the document itself
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    sha256: Option<[u8; 32]>,
+    pub sha256: Option<[u8; 32]>,
     /// The sha1 of the document itself
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    sha1: Option<[u8; 20]>,
+    pub sha1: Option<[u8; 20]>,
     /// The signatures listed on the document.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    signatures: Vec<Signature>,
+    pub signatures: Vec<Signature>,
 }
 
 /// A shared random value produced by the directory authorities.
@@ -432,65 +524,46 @@ pub struct SharedRandVal([u8; 32]);
 
 /// A shared-random value produced by the directory authorities,
 /// along with meta-information about that value.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
+#[non_exhaustive]
+#[cfg_attr(feature = "parse2", derive(Deftly), derive_deftly(ItemValueParseable))]
 pub struct SharedRandStatus {
     /// How many authorities revealed shares that contributed to this value.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    n_reveals: u8,
+    pub n_reveals: u8,
     /// The current random value.
     ///
     /// The properties of the secure shared-random system guarantee
     /// that this value isn't predictable before it first becomes
     /// live, and that a hostile party could not have forced it to
     /// have any more than a small number of possible random values.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    value: SharedRandVal,
+    pub value: SharedRandVal,
 
     /// The time when this SharedRandVal becomes (or became) the latest.
     ///
     /// (This is added per proposal 342, assuming that gets accepted.)
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    timestamp: Option<time::SystemTime>,
+    pub timestamp: Option<Iso8601TimeNoSp>,
 }
 
 /// Description of an authority's identity and address.
 ///
 /// (Corresponds to a dir-source line.)
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
-struct DirSource {
+#[non_exhaustive]
+pub struct DirSource {
     /// human-readable nickname for this authority.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    nickname: String,
+    pub nickname: String,
     /// Fingerprint for the _authority_ identity key of this
     /// authority.
     ///
     /// This is the same key as the one that signs the authority's
     /// certificates.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    identity: RsaIdentity,
+    pub identity: RsaIdentity,
     /// IP address for the authority
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    ip: net::IpAddr,
+    pub ip: net::IpAddr,
     /// HTTP directory port for this authority
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    dir_port: u16,
+    pub dir_port: u16,
     /// OR port for this authority.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    or_port: u16,
+    pub or_port: u16,
 }
 
 bitflags! {
@@ -578,44 +651,28 @@ impl RelayWeight {
 }
 
 /// All information about a single authority, as represented in a consensus
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
-pub(crate) struct ConsensusVoterInfo {
+#[non_exhaustive]
+pub struct ConsensusVoterInfo {
     /// Contents of the dirsource line about an authority
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    dir_source: DirSource,
+    pub dir_source: DirSource,
     /// Human-readable contact information about the authority
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    contact: String,
+    pub contact: String,
     /// Digest of the vote that the authority cast to contribute to
     /// this consensus.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    vote_digest: Vec<u8>,
+    pub vote_digest: Vec<u8>,
 }
 
 /// The signed footer of a consensus netstatus.
-#[allow(dead_code)]
-#[cfg_attr(
-    feature = "dangerous-expose-struct-fields",
-    visible::StructFields(pub),
-    visibility::make(pub),
-    non_exhaustive
-)]
 #[derive(Debug, Clone)]
-pub(crate) struct Footer {
+#[non_exhaustive]
+pub struct Footer {
     /// Weights to be applied to certain classes of relays when choosing
     /// for different roles.
     ///
     /// For example, we want to avoid choosing exits for non-exit
     /// roles when overall the proportion of exits is small.
-    #[cfg_attr(docsrs, doc(cfg(feature = "dangerous-expose-struct-fields")))]
-    weights: NetParams<i32>,
+    pub weights: NetParams<i32>,
 }
 
 /// A consensus document that lists relays along with their
@@ -916,6 +973,21 @@ where
     }
 }
 
+impl FromStr for SharedRandVal {
+    type Err = Error;
+    fn from_str(s: &str) -> Result<Self> {
+        let val: B64 = s.parse()?;
+        let val = SharedRandVal(val.into_array()?);
+        Ok(val)
+    }
+}
+impl Display for SharedRandVal {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        Display::fmt(&B64::from(Vec::from(self.0)), f)
+    }
+}
+impl NormalItemArgument for SharedRandVal {}
+
 impl SharedRandStatus {
     /// Parse a current or previous shared rand value from a given
     /// SharedRandPreviousValue or SharedRandCurrentValue.
@@ -931,12 +1003,9 @@ impl SharedRandStatus {
             }
         }
         let n_reveals: u8 = item.parse_arg(0)?;
-        let val: B64 = item.parse_arg(1)?;
-        let value = SharedRandVal(val.into_array()?);
+        let value: SharedRandVal = item.parse_arg(1)?;
         // Added in proposal 342
-        let timestamp = item
-            .parse_optional_arg::<Iso8601TimeNoSp>(2)?
-            .map(Into::into);
+        let timestamp = item.parse_optional_arg::<Iso8601TimeNoSp>(2)?;
         Ok(SharedRandStatus {
             n_reveals,
             value,
@@ -951,7 +1020,7 @@ impl SharedRandStatus {
 
     /// Return the timestamp (if any) associated with this `SharedRandValue`.
     pub fn timestamp(&self) -> Option<std::time::SystemTime> {
-        self.timestamp
+        self.timestamp.map(|t| t.0)
     }
 }
 
@@ -1031,6 +1100,45 @@ impl std::str::FromStr for RelayFlags {
     }
 }
 
+/// Parsing helper for a relay flags line (eg `s` item in a routerdesc)
+struct RelayFlagsParser<'s> {
+    /// Flags so far, including the implied ones
+    flags: RelayFlags,
+
+    /// The previous argument, if any
+    ///
+    /// Used only for checking that the arguments are sorted, as per the spec.
+    prev: Option<&'s str>,
+}
+
+impl<'s> RelayFlagsParser<'s> {
+    /// Start parsing relay flags
+    fn new() -> Self {
+        // These flags are implicit.
+        RelayFlagsParser {
+            flags: RelayFlags::RUNNING | RelayFlags::VALID,
+            prev: None,
+        }
+    }
+    /// Parse the next relay flag argument
+    fn add(&mut self, arg: &'s str) -> StdResult<(), &'static str> {
+        if let Some(prev) = self.prev {
+            if prev >= arg {
+                // Arguments out of order.
+                return Err("Flags out of order");
+            }
+        }
+        let fl = arg.parse().void_unwrap();
+        self.flags |= fl;
+        self.prev = Some(arg);
+        Ok(())
+    }
+    /// Finish parsing relay flags
+    fn finish(self) -> RelayFlags {
+        self.flags
+    }
+}
+
 impl RelayFlags {
     /// Parse a relay-flags entry from an "s" line.
     fn from_item(item: &Item<'_, NetstatusKwd>) -> Result<RelayFlags> {
@@ -1040,25 +1148,15 @@ impl RelayFlags {
                     .at_pos(item.pos()),
             );
         }
-        // These flags are implicit.
-        let mut flags: RelayFlags = RelayFlags::RUNNING | RelayFlags::VALID;
+        let mut flags = RelayFlagsParser::new();
 
-        let mut prev: Option<&str> = None;
         for s in item.args() {
-            if let Some(p) = prev {
-                if p >= s {
-                    // Arguments out of order.
-                    return Err(EK::BadArgument
-                        .at_pos(item.pos())
-                        .with_msg("Flags out of order"));
-                }
-            }
-            let fl = s.parse().void_unwrap();
-            flags |= fl;
-            prev = Some(s);
+            flags
+                .add(s)
+                .map_err(|msg| EK::BadArgument.at_pos(item.pos()).with_msg(msg))?;
         }
 
-        Ok(flags)
+        Ok(flags.finish())
     }
 }
 
@@ -1078,8 +1176,15 @@ impl RelayWeight {
             );
         }
 
-        let params: NetParams<u32> = item.args_as_str().parse()?;
+        let params = item.args_as_str().parse()?;
 
+        Self::from_net_params(&params).map_err(|e| e.at_pos(item.pos()))
+    }
+
+    /// Parse a routerweight from partially-parsed `w` line in the form of a `NetParams`
+    ///
+    /// This function is the common part shared between `parse2` and `parse`.
+    fn from_net_params(params: &NetParams<u32>) -> Result<RelayWeight> {
         let bw = params.params.get("Bandwidth");
         let unmeas = params.params.get("Unmeasured");
 
@@ -1091,9 +1196,62 @@ impl RelayWeight {
         match unmeas {
             None | Some(0) => Ok(RelayWeight::Measured(bw)),
             Some(1) => Ok(RelayWeight::Unmeasured(bw)),
-            _ => Err(EK::BadArgument
-                .at_pos(item.pos())
-                .with_msg("unmeasured value")),
+            _ => Err(EK::BadArgument.with_msg("unmeasured value")),
+        }
+    }
+}
+
+/// `parse2` impls for types in this module
+///
+/// Separate module to save on repeated `cfg` and for a separate namespace.
+#[cfg(feature = "parse2")]
+mod parse2_impls {
+    use super::*;
+    use parse2::ArgumentError as AE;
+    use parse2::ErrorProblem as EP;
+    use parse2::{ArgumentStream, ItemArgumentParseable, ItemValueParseable};
+    use std::result::Result;
+
+    impl ItemValueParseable for RelayWeight {
+        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            (|| {
+                let params = item.args_copy().into_remaining().parse()?;
+                Self::from_net_params(&params)
+            })()
+            .map_err(item.invalid_argument_handler("weights"))
+        }
+    }
+
+    impl ItemValueParseable for RelayFlags {
+        fn from_unparsed(item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            let mut flags = RelayFlagsParser::new();
+            for arg in item.args_copy() {
+                flags
+                    .add(arg)
+                    .map_err(item.invalid_argument_handler("flags"))?;
+            }
+            Ok(flags.finish())
+        }
+    }
+
+    impl ItemValueParseable for rs::Version {
+        fn from_unparsed(mut item: parse2::UnparsedItem<'_>) -> Result<Self, EP> {
+            item.check_no_object()?;
+            item.args_mut()
+                .into_remaining()
+                .parse()
+                .map_err(item.invalid_argument_handler("version"))
+        }
+    }
+
+    impl ItemArgumentParseable for IgnoredPublicationTimeSp {
+        fn from_args(a: &mut ArgumentStream) -> Result<IgnoredPublicationTimeSp, AE> {
+            let mut next_arg = || a.next().ok_or(AE::Missing);
+            let _: &str = next_arg()?;
+            let _: &str = next_arg()?;
+            Ok(IgnoredPublicationTimeSp)
         }
     }
 }
@@ -1298,6 +1456,11 @@ mod test {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
     use hex_literal::hex;
+    #[cfg(all(feature = "ns-vote", feature = "parse2"))]
+    use {
+        crate::parse2::{NetdocSigned as _, parse_netdoc},
+        std::fs,
+    };
 
     const CERTS: &str = include_str!("../../testdata/authcerts2.txt");
     const CONSENSUS: &str = include_str!("../../testdata/mdconsensus1.txt");
@@ -1378,8 +1541,8 @@ mod test {
         assert!(!pv.supports_subver("HSDir", 3));
         let ip4 = "127.0.0.1:5002".parse::<SocketAddr>().unwrap();
         let ip6 = "[::1]:5002".parse::<SocketAddr>().unwrap();
-        assert!(r0.orport_addrs().any(|a| a == &ip4));
-        assert!(r0.orport_addrs().any(|a| a == &ip6));
+        assert!(r0.addrs().any(|a| a == ip4));
+        assert!(r0.addrs().any(|a| a == ip6));
 
         Ok(())
     }
@@ -1410,6 +1573,23 @@ mod test {
         Ok(())
     }
 
+    #[test]
+    #[cfg(all(feature = "ns-vote", feature = "parse2"))]
+    fn parse2_vote() -> anyhow::Result<()> {
+        let file = "testdata2/v3-status-votes--1";
+        let text = fs::read_to_string(file)?;
+
+        // TODO replace the poc struct here when we have parsing of proper whole votes
+        use crate::parse2::poc::netstatus::NetworkStatusSignedVote;
+
+        let doc: NetworkStatusSignedVote = parse_netdoc(&text, file)?;
+
+        println!("{doc:?}");
+        println!("{:#?}", doc.inspect_unverified().0.r[0]);
+
+        Ok(())
+    }
+
     #[test]
     fn test_bad() {
         use crate::Pos;
@@ -1535,7 +1715,7 @@ mod test {
         assert_eq!(sr2.n_reveals, sr.n_reveals);
         assert_eq!(sr2.value.0, sr.value.0);
         assert_eq!(
-            sr2.timestamp.unwrap(),
+            sr2.timestamp.unwrap().0,
             humantime::parse_rfc3339("2022-01-20T12:34:56Z").unwrap()
         );
 
