@@ -279,6 +279,15 @@ pub enum NetdocErrorKind {
     /// Found an empty line in the middle of a document
     #[display("Empty line")]
     EmptyLine,
+    /// The document began with a deprecated unicode BOM marker.
+    #[display("unexpected byte-order marker")]
+    BomMarkerFound,
+    /// The document contained an internal NUL byte
+    #[display("unexpected NUL")]
+    NulFound,
+    /// An item contained extra spaces at a place where they are not allowed.
+    #[display("Extraneous spaces")]
+    ExtraneousSpace,
 }
 
 /// The underlying source for an [`Error`](struct@Error).
