@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,11 +41,15 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
 #![cfg_attr(not(all(feature = "full", feature = "experimental")), allow(unused))]
 
+#[cfg(feature = "parse2")]
+#[macro_use]
+pub mod parse2;
 #[cfg(feature = "hs-service")]
 pub(crate) mod build;
 #[macro_use]
@@ -55,6 +59,9 @@ mod err;
 pub mod types;
 mod util;
 
+#[doc(hidden)]
+pub use derive_deftly;
+
 // Use `#[doc(hidden)]` rather than pub(crate), because otherwise the doctest
 // doesn't work.
 #[doc(hidden)]
@@ -88,6 +95,35 @@ pub enum AllowAnnotations {
     AnnotationsNotAllowed,
 }
 
+/// A "normally formatted" argument to a netdoc item
+///
+/// A type that is represented as a single argument
+/// whose representation is as for the type's `FromStr` and `Display`.
+///
+/// Implementing this trait enables a blanket impl of `parse2::ItemArgumentParseable`
+/// and `build::ItemArgument`.
+pub trait NormalItemArgument: std::str::FromStr + std::fmt::Display {}
+// TODO: should we implement ItemArgument for, say, tor_llcrypto::pk::rsa::RsaIdentity ?
+// It's not clear whether it's always formatted the same way in all parts of the spec.
+// The Display impl of RsaIdentity adds a `$` which is not supposed to be present
+// in (for example) an authority certificate (authcert)'s "fingerprint" line.
+
+impl NormalItemArgument for usize {}
+impl NormalItemArgument for u8 {}
+impl NormalItemArgument for u16 {}
+impl NormalItemArgument for u32 {}
+impl NormalItemArgument for u64 {}
+impl NormalItemArgument for u128 {}
+
+impl NormalItemArgument for isize {}
+impl NormalItemArgument for i8 {}
+impl NormalItemArgument for i16 {}
+impl NormalItemArgument for i32 {}
+impl NormalItemArgument for i64 {}
+impl NormalItemArgument for i128 {}
+
+impl NormalItemArgument for String {}
+
 /// Return a list of the protocols [supported](tor_protover::doc_supported)
 /// by this crate.
 pub fn supported_protocols() -> tor_protover::Protocols {
