@@ -1,7 +1,7 @@
 //! Test helpers.
 
 #[cfg(test)]
-use super::{rtt::RoundtripTimeEstimator, CongestionWindow};
+use super::{CongestionWindow, rtt::RoundtripTimeEstimator};
 
 // Make a new RTT estimator.
 #[cfg(test)]
@@ -12,7 +12,7 @@ pub(crate) fn new_rtt_estimator() -> RoundtripTimeEstimator {
 // Make a new congestion window.
 #[cfg(test)]
 pub(crate) fn new_cwnd() -> CongestionWindow {
-    CongestionWindow::new(&params::build_cwnd_params())
+    CongestionWindow::new(params::build_cwnd_params())
 }
 
 #[cfg(test)]
@@ -48,8 +48,8 @@ pub(crate) mod params {
         CongestionControlParamsBuilder::default()
             .rtt_params(build_rtt_params())
             .cwnd_params(build_cwnd_params())
-            .alg(Algorithm::Vegas(params.clone()))
-            .fallback_alg(Algorithm::FixedWindow(build_fixed_params()))
+            .alg(Algorithm::Vegas(params))
+            .fixed_window_params(build_fixed_params())
             .build()
             .expect("Unable to build CC params")
     }
@@ -59,8 +59,8 @@ pub(crate) mod params {
         CongestionControlParamsBuilder::default()
             .rtt_params(build_rtt_params())
             .cwnd_params(build_cwnd_params())
-            .alg(Algorithm::FixedWindow(params.clone()))
-            .fallback_alg(Algorithm::FixedWindow(params))
+            .alg(Algorithm::FixedWindow(params))
+            .fixed_window_params(params)
             .build()
             .expect("Unable to build CC params")
     }
