@@ -124,7 +124,7 @@ pub(crate) trait InboundRelayLayer {
 #[allow(dead_code)]
 #[cfg_attr(feature = "bench", visibility::make(pub))]
 pub(crate) trait OutboundRelayLayer {
-    /// Decrypt a RelayCellBody that is moving towards the client.
+    /// Decrypt a RelayCellBody that is coming from the client.
     ///
     /// Return an authentication tag if it is addressed to us.
     fn decrypt_outbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> Option<SendmeTag>;
@@ -156,7 +156,7 @@ pub(crate) trait InboundClientLayer {
 /// Type to store hop indices on a circuit.
 ///
 /// Hop indices are zero-based: "0" denotes the first hop on the circuit.
-#[derive(Copy, Clone, Eq, PartialEq, Debug, Deftly)]
+#[derive(Copy, Clone, Eq, PartialEq, Debug, Deftly, Ord, PartialOrd)]
 #[derive_deftly(HasMemoryCost)]
 pub struct HopNum(u8);
 
@@ -171,6 +171,11 @@ impl HopNum {
     pub fn display(&self) -> HopNumDisplay {
         HopNumDisplay(*self)
     }
+
+    /// Return true if this is  the first hop of a circuit.
+    pub(crate) fn is_first_hop(&self) -> bool {
+        self.0 == 0
+    }
 }
 
 /// A helper for displaying [`HopNum`]s.
@@ -321,6 +326,14 @@ pub(crate) type Tor1RelayCrypto =
 pub(crate) type Tor1Hsv3RelayCrypto =
     tor1::CryptStatePair<tor_llcrypto::cipher::aes::Aes256Ctr, tor_llcrypto::d::Sha3_256>;
 
+/// Counter galois onion relay crypto.
+//
+// We use `aes` directly here instead of tor_llcrypto::aes, which may or may not be OpenSSL:
+// the OpenSSL implementations have bad performance when it comes to re-keying
+// or changing IVs.
+#[cfg(feature = "counter-galois-onion")]
+pub(crate) type CgoRelayCrypto = cgo::CryptStatePair<aes::Aes128, aes::Aes128Enc>;
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -338,8 +351,8 @@ mod test {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
     use super::*;
-    use rand::{seq::IndexedRandom as _, RngCore};
-    use tor_basic_utils::{test_rng::testing_rng, RngExt as _};
+    use rand::{RngCore, seq::IndexedRandom as _};
+    use tor_basic_utils::{RngExt as _, test_rng::testing_rng};
     use tor_bytes::SecretBuf;
     use tor_cell::relaycell::RelayCellFormat;
 
