@@ -1,10 +1,14 @@
 //! Define an error type for the tor-proto crate.
+use safelog::Sensitive;
 use std::{sync::Arc, time::Duration};
 use thiserror::Error;
 use tor_cell::relaycell::{StreamId, msg::EndReason};
 use tor_error::{Bug, ErrorKind, HasKind};
 use tor_linkspec::RelayIdType;
 
+use crate::HopNum;
+use crate::client::circuit::PathEntry;
+
 /// An error type for the tor-proto crate.
 ///
 /// This type should probably be split into several.  There's more
@@ -115,6 +119,19 @@ pub enum Error {
     /// Received a cell with a stream ID of zero.
     #[error("Received a cell with a stream ID of zero")]
     StreamIdZero,
+    /// Received a cell on a closed or non-existent stream.
+    #[error(
+        "Received a cell from {} on a closed or non-existent stream {}. \
+         Either they are violating the protocol, or we are expiring streams too aggressively",
+        src,
+        streamid
+    )]
+    UnknownStream {
+        /// The hop the cell originated from.
+        src: Sensitive<PathEntry>,
+        /// The stream ID of the cell.
+        streamid: StreamId,
+    },
     /// Couldn't extend a circuit because the extending relay or the
     /// target relay refused our request.
     #[error("Circuit extension refused: {0}")]
@@ -139,6 +156,10 @@ pub enum Error {
     #[error("Tried to send too many outbound cells")]
     ExcessOutboundCells,
 
+    /// Received unexpected or excessive inbound circuit
+    #[error("Received unexpected or excessive circuit padding from {}", _1.display())]
+    ExcessPadding(#[source] ExcessPadding, HopNum),
+
     /// Channel does not match target
     #[error("Peer identity mismatch: {0}")]
     ChanMismatch(String),
@@ -246,8 +267,10 @@ impl From<Error> for std::io::Error {
             | MissingId(_)
             | IdUnavailable(_)
             | StreamIdZero
+            | UnknownStream { .. }
             | ExcessInboundCells
-            | ExcessOutboundCells => ErrorKind::InvalidData,
+            | ExcessOutboundCells
+            | ExcessPadding(_, _) => ErrorKind::InvalidData,
 
             Bug(ref e) if e.kind() == tor_error::ErrorKind::BadApiUsage => ErrorKind::InvalidData,
 
@@ -297,8 +320,10 @@ impl HasKind for Error {
             E::MissingId(_) => EK::BadApiUsage,
             E::IdUnavailable(_) => EK::BadApiUsage,
             E::StreamIdZero => EK::BadApiUsage,
+            E::UnknownStream { .. } => EK::TorProtocolViolation,
             E::ExcessInboundCells => EK::TorProtocolViolation,
             E::ExcessOutboundCells => EK::Internal,
+            E::ExcessPadding(_, _) => EK::TorProtocolViolation,
             E::Memquota(err) => err.kind(),
             E::Bug(e) => e.kind(),
         }
@@ -356,3 +381,14 @@ pub(crate) enum ConfluxHandshakeError {
     /// The channel was closed.
     ChannelClosed,
 }
+
+#[derive(Debug, Clone, Error)]
+#[non_exhaustive]
+pub enum ExcessPadding {
+    /// We received circuit padding when we hadn't negotiated a padding framework.
+    #[error("Padding received when not negotiated with given hop")]
+    NoPaddingNegotiated,
+    /// We received more padding than our negotiated framework permits.
+    #[error("Received padding in excess of negotiated framework's limit")]
+    PaddingExceedsLimit,
+}
