@@ -4,12 +4,11 @@
 //! version 1, and communicate with them in order to specify configuration parameters and
 //! receive updates as to the current state of the PT.
 
+use crate::PtClientMethod;
 use crate::err;
 use crate::err::PtError;
-use crate::PtClientMethod;
-use futures::channel::mpsc;
-use futures::channel::mpsc::Receiver;
 use futures::StreamExt;
+use futures::channel::mpsc::Receiver;
 use itertools::Itertools;
 use std::borrow::Cow;
 use std::collections::HashMap;
@@ -69,6 +68,7 @@ pub enum PtMessage {
         /// The protocol used ('socks4' or 'socks5').
         protocol: String,
         /// An address to connect via this transport.
+        /// (This should be localhost.)
         endpoint: SocketAddr,
     },
     /// `CMETHOD-ERROR`: An error was encountered setting up a client transport.
@@ -227,6 +227,11 @@ impl FromStr for PtMessage {
                     .ok_or_else(|| Cow::from("no endpoint"))?
                     .parse::<SocketAddr>()
                     .map_err(|e| Cow::from(format!("failed to parse endpoint: {}", e)))?;
+                if !endpoint.ip().is_loopback() {
+                    return Err(Cow::from(format!(
+                        "CMETHOD endpoint {endpoint} was not localhost"
+                    )));
+                }
                 Self::ClientTransportLaunched {
                     transport: transport
                         .parse()
@@ -256,6 +261,7 @@ impl FromStr for PtMessage {
                     .ok_or_else(|| Cow::from("no endpoint"))?
                     .parse::<SocketAddr>()
                     .map_err(|e| Cow::from(format!("failed to parse endpoint: {}", e)))?;
+                // The SMETHOD endpoint is the place where _clients_ connect, and it shouldn't be localhost.
                 let mut parsed_args = HashMap::new();
 
                 // NOTE(eta): pt-spec.txt seems to imply these options can't contain spaces, so
@@ -380,7 +386,8 @@ pub(crate) mod sealed {
                     PtError::Internal(internal!("Created child process without stdout pipe"))
                 })?,
             );
-            let (mut tx, rx) = mpsc::channel(PT_STDIO_BUFFER);
+            // TODO RELAY #1649 We don't use a tor_memquota::mq_queue here yet
+            let (mut tx, rx) = tor_async_utils::mpsc_channel_no_memquota(PT_STDIO_BUFFER);
             let ident = identifier.clone();
             #[allow(clippy::cognitive_complexity)]
             thread::spawn(move || {
@@ -456,6 +463,7 @@ pub(crate) mod sealed {
         /// Receive a message from the pluggable transport binary asynchronously.
         ///
         /// Note: This will convert `PtMessage::Log` into a tracing log call automatically.
+        #[allow(clippy::cognitive_complexity)] // due to tracing
         pub async fn recv(&mut self) -> err::Result<PtMessage> {
             loop {
                 match self.stdout.next().await {
@@ -527,7 +535,7 @@ pub(crate) mod sealed {
                         return Err(PtError::ProtocolViolation(format!(
                             "unknown CMETHOD protocol '{}'",
                             x
-                        )))
+                        )));
                     }
                 },
                 None => SocksVersion::V5,
