@@ -3,7 +3,7 @@
 #[cfg(feature = "geoip")]
 use tor_geoip::HasCountryCode;
 use tor_linkspec::{ChanTarget, HasAddrs, HasRelayIds, RelayIdSet};
-use tor_netdir::{NetDir, Relay, SubnetConfig};
+use tor_netdir::{FamilyRules, NetDir, Relay, SubnetConfig};
 use tor_netdoc::types::policy::AddrPortPattern;
 
 use crate::{LowLevelRelayPredicate, RelaySelectionConfig, RelayUsage};
@@ -157,8 +157,7 @@ fn relay_has_addr_in_set(relay: &Relay<'_>, patterns: &[AddrPortPattern]) -> boo
     // where the transport doesn't use an address.
     relay
         .addrs()
-        .iter()
-        .any(|addr| patterns.iter().any(|pat| pat.matches_sockaddr(addr)))
+        .any(|addr| patterns.iter().any(|pat| pat.matches_sockaddr(&addr)))
 }
 
 /// A set of relays that we must not use when picking a given
@@ -187,6 +186,8 @@ pub struct RelayExclusion<'a> {
     /// The configuration to use when deciding whether two addresses are in the
     /// same subnet.
     subnet_config: SubnetConfig,
+    /// The rules to use when deciding whether two relays are in the same family.
+    family_rules: FamilyRules,
 }
 
 /// Helper: wraps `Vec[Relay]`, but implements Debug.
@@ -216,6 +217,7 @@ impl<'a> RelayExclusion<'a> {
             exclude_subnets: Vec::new(),
             exclude_relay_families: RelayList(Vec::new()),
             subnet_config: SubnetConfig::no_addresses_match(),
+            family_rules: FamilyRules::ignore_declared_families(),
         }
     }
 
@@ -254,11 +256,15 @@ impl<'a> RelayExclusion<'a> {
         netdir: &'a NetDir,
     ) -> Self {
         if let Some(r) = netdir.by_ids(ct) {
-            return Self::exclude_relays_in_same_family(cfg, vec![r]);
+            return Self::exclude_relays_in_same_family(
+                cfg,
+                vec![r],
+                FamilyRules::from(netdir.params()),
+            );
         }
 
         let exclude_ids = ct.identities().map(|id_ref| id_ref.to_owned()).collect();
-        let exclude_addr_families = ct.addrs().iter().map(|a| a.ip()).collect();
+        let exclude_addr_families = ct.addrs().map(|a| a.ip()).collect();
 
         Self {
             exclude_ids,
@@ -280,10 +286,12 @@ impl<'a> RelayExclusion<'a> {
     pub fn exclude_relays_in_same_family(
         cfg: &RelaySelectionConfig,
         relays: Vec<Relay<'a>>,
+        family_rules: FamilyRules,
     ) -> Self {
         RelayExclusion {
             exclude_relay_families: RelayList(relays),
             subnet_config: cfg.subnet_config,
+            family_rules,
             ..RelayExclusion::no_relays_excluded()
         }
     }
@@ -298,6 +306,7 @@ impl<'a> RelayExclusion<'a> {
             exclude_subnets: exclude_addr_families,
             exclude_relay_families,
             subnet_config,
+            family_rules,
         } = other;
         self.exclude_ids
             .extend(exclude_ids.iter().map(|id_ref| id_ref.to_owned()));
@@ -307,6 +316,7 @@ impl<'a> RelayExclusion<'a> {
             .0
             .extend_from_slice(&exclude_relay_families.0[..]);
         self.subnet_config = self.subnet_config.union(subnet_config);
+        self.family_rules = self.family_rules.union(family_rules);
     }
 
     /// Return a string describing why we rejected the relays that _don't_ match
@@ -330,7 +340,7 @@ impl<'a> LowLevelRelayPredicate for RelayExclusion<'a> {
             return false;
         }
 
-        if relay.addrs().iter().any(|addr| {
+        if relay.addrs().any(|addr| {
             self.exclude_subnets
                 .iter()
                 .any(|fam| self.subnet_config.addrs_in_same_subnet(&addr.ip(), fam))
@@ -338,12 +348,9 @@ impl<'a> LowLevelRelayPredicate for RelayExclusion<'a> {
             return false;
         }
 
-        if self
-            .exclude_relay_families
-            .0
-            .iter()
-            .any(|r| relays_in_same_extended_family(&self.subnet_config, relay, r))
-        {
+        if self.exclude_relay_families.0.iter().any(|r| {
+            relays_in_same_extended_family(&self.subnet_config, relay, r, self.family_rules)
+        }) {
             return false;
         }
 
@@ -358,8 +365,10 @@ fn relays_in_same_extended_family(
     subnet_config: &SubnetConfig,
     r1: &Relay<'_>,
     r2: &Relay<'_>,
+    family_rules: FamilyRules,
 ) -> bool {
-    r1.in_same_family(r2) || subnet_config.any_addrs_in_same_subnet(r1, r2)
+    r1.low_level_details().in_same_family(r2, family_rules)
+        || subnet_config.any_addrs_in_same_subnet(r1, r2)
 }
 
 #[cfg(test)]
@@ -379,6 +388,7 @@ mod test {
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
     use tor_linkspec::RelayId;
+    use tor_netdir::testnet::construct_custom_netdir;
 
     use super::*;
     use crate::testing::{cfg, split_netdir, testnet};
@@ -387,9 +397,10 @@ mod test {
     fn exclude_nothing() {
         let nd = testnet();
         let usage = RelayExclusion::no_relays_excluded();
-        assert!(nd
-            .relays()
-            .all(|r| usage.low_level_predicate_permits_relay(&r)));
+        assert!(
+            nd.relays()
+                .all(|r| usage.low_level_predicate_permits_relay(&r))
+        );
     }
 
     #[test]
@@ -423,16 +434,16 @@ mod test {
             &nd,
             &RelayExclusion::exclude_specific_relays(&[relay_0.clone(), relay_5.clone()]),
         );
-        let p = |r: &Relay<'_>| !(r.same_relay(&relay_0) || r.same_relay(&relay_5));
+        let p = |r: &Relay<'_>| !(r.same_relay_ids(&relay_0) || r.same_relay_ids(&relay_5));
         assert_eq!(yes.len(), 38);
         assert_eq!(no.len(), 2);
         assert!(yes.iter().all(p));
         assert!(no.iter().all(|r| !p(r)));
     }
 
-    #[test]
-    fn exclude_families() {
-        let nd = testnet();
+    /// Helper for testing family exclusions.  Requires a netdir where,
+    /// for every N, relays 2N and 2N+1 are in a family.
+    fn exclude_families_impl(nd: &NetDir, family_rules: FamilyRules) {
         let id_0: RelayId = "$0000000000000000000000000000000000000000".parse().unwrap();
         let id_5: RelayId = "ed25519:BQUFBQUFBQUFBQUFBQUFBQUFBQUFBQUFBQUFBQUFBQU"
             .parse()
@@ -454,10 +465,11 @@ mod test {
         };
 
         let (yes, no) = split_netdir(
-            &nd,
+            nd,
             &RelayExclusion::exclude_relays_in_same_family(
                 &cfg_no_subnet,
                 excluding_relays.clone(),
+                family_rules,
             ),
         );
         let p = |r: &Relay<'_>| !r.identities().any(|id| expect_excluded_ids.contains(id));
@@ -486,8 +498,8 @@ mod test {
             .collect();
 
         let (yes, no) = split_netdir(
-            &nd,
-            &RelayExclusion::exclude_relays_in_same_family(&cfg(), excluding_relays),
+            nd,
+            &RelayExclusion::exclude_relays_in_same_family(&cfg(), excluding_relays, family_rules),
         );
         for r in &no {
             dbg!(r.rsa_identity().unwrap());
@@ -502,6 +514,37 @@ mod test {
         assert!(no.iter().all(|r| { !p(r) }));
     }
 
+    #[test]
+    fn exclude_families_by_list() {
+        exclude_families_impl(
+            &testnet(),
+            *FamilyRules::ignore_declared_families().use_family_lists(true),
+        );
+    }
+
+    #[test]
+    fn exclude_families_by_id() {
+        // Here we construct a network that matches our default testnet,
+        // but without any family lists.
+        // Instead we use "happy family" IDs to match the families from that default testnest.
+        let netdir = construct_custom_netdir(|pos, nb, _| {
+            // Clear the family list.
+            nb.md.family("".parse().unwrap());
+            // This will create an "Unrecognized" family id such that
+            // pos:N  will be shared by nodes in positions 2N and 2N+1.
+            let fam_id = format!("pos:{}", pos / 2);
+            nb.md.add_family_id(fam_id.parse().unwrap());
+        })
+        .unwrap()
+        .unwrap_if_sufficient()
+        .unwrap();
+
+        exclude_families_impl(
+            &netdir,
+            *FamilyRules::ignore_declared_families().use_family_ids(true),
+        );
+    }
+
     #[test]
     fn filter_addresses() {
         let nd = testnet();
@@ -516,7 +559,7 @@ mod test {
         assert_eq!(no.len(), 24);
 
         let expected = ["1.0.0.3".parse().unwrap(), "2.0.0.3".parse().unwrap()];
-        let p = |r: &Relay<'_>| expected.contains(&r.addrs()[0].ip());
+        let p = |r: &Relay<'_>| expected.contains(&r.addrs().next().unwrap().ip());
         assert!(yes.iter().all(p));
         assert!(no.iter().all(|r| !p(r)));
     }
