@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,6 +41,7 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 pub mod dispatch;
@@ -48,13 +49,13 @@ mod err;
 mod method;
 mod obj;
 
-use std::{convert::Infallible, sync::Arc};
+use std::{collections::HashSet, convert::Infallible, sync::Arc};
 
 pub use dispatch::{DispatchTable, InvokeError, UpdateSink};
 pub use err::{RpcError, RpcErrorKind};
 pub use method::{
-    check_method_names, is_method_name, iter_method_names, DeserMethod, DynMethod,
-    InvalidMethodName, Method, NoUpdates, RpcMethod,
+    DeserMethod, DynMethod, Method, NoUpdates, RpcMethod, check_method_names, is_method_name,
+    iter_method_names,
 };
 pub use obj::{Object, ObjectArcExt, ObjectId};
 
@@ -72,7 +73,7 @@ pub use obj::cast::CastTable;
 #[doc(hidden)]
 pub use {
     derive_deftly, dispatch::RpcResult, downcast_rs, erased_serde, futures, inventory,
-    method::MethodInfo_, once_cell, paste, tor_async_utils, tor_error::internal, typetag,
+    method::MethodInfo_, paste, tor_async_utils, tor_error::internal, typetag,
 };
 
 /// Templates for use with [`derive_deftly`]
@@ -116,29 +117,14 @@ pub trait Context: Send + Sync {
     /// Create an owning reference to `object` within this context.
     ///
     /// Return an ObjectId for this object.
-    ///
-    /// TODO RPC: We may need to change the above semantics and the name of this
-    /// function depending on how we decide to name and specify things.
     fn register_owned(&self, object: Arc<dyn Object>) -> ObjectId;
 
-    /// Make sure that
-    /// this context contains a non-owning reference to `object`,
-    /// creating one if necessary.
-    ///
-    /// Return an ObjectId for this object.
-    ///
-    /// Note that this takes an Arc, since that's required in order to find a
-    /// working type Id for the target object.
-    ///
-    /// TODO RPC: We may need to change the above semantics and the name of this
-    /// function depending on how we decide to name and specify things.
-    fn register_weak(&self, object: Arc<dyn Object>) -> ObjectId;
+    // TODO: If we add weak references again, we may need a register_weak method here.
 
     /// Drop an owning reference to the object called `object` within this context.
     ///
-    /// This will return an error if `object` is not an owning reference.
-    ///
-    /// TODO RPC should this really return a LookupError?
+    /// This will return an error if `object` is not an owning reference,
+    /// or does not exist.
     fn release_owned(&self, object: &ObjectId) -> Result<(), LookupError>;
 
     /// Return a dispatch table that can be used to invoke other RPC methods.
@@ -270,6 +256,59 @@ pub struct SingleIdResponse {
     id: ObjectId,
 }
 
+/// Error representing an "invalid" RPC identifier.
+#[derive(Clone, Debug, thiserror::Error)]
+#[non_exhaustive]
+#[cfg_attr(test, derive(Eq, PartialEq))]
+pub enum InvalidRpcIdentifier {
+    /// The method doesn't have a ':' to demarcate its namespace.
+    #[error("Identifier has no namespace separator")]
+    NoNamespace,
+
+    /// The method's namespace is not one we recognize.
+    #[error("Identifier has unrecognized namespace")]
+    UnrecognizedNamespace,
+
+    /// The method's name is not in snake_case.
+    #[error("Identifier name has unexpected format")]
+    BadIdName,
+}
+
+/// Check whether `method` is an expected and well-formed RPC identifier.
+///
+/// If `recognized_namespaces` is provided, only identifiers within those
+/// namespaces are accepted; otherwise, all namespaces are accepted.
+///
+/// (Examples of RPC identifiers are method names.)
+pub(crate) fn is_valid_rpc_identifier(
+    recognized_namespaces: Option<&HashSet<&str>>,
+    method: &str,
+) -> Result<(), InvalidRpcIdentifier> {
+    /// Return true if name is in acceptable format.
+    fn name_ok(n: &str) -> bool {
+        let mut chars = n.chars();
+        let Some(first) = chars.next() else {
+            return false;
+        };
+        first.is_ascii_lowercase()
+            && chars.all(|ch| ch.is_ascii_lowercase() || ch.is_ascii_digit() || ch == '_')
+    }
+    let (scope, name) = method
+        .split_once(':')
+        .ok_or(InvalidRpcIdentifier::NoNamespace)?;
+
+    if let Some(recognized_namespaces) = recognized_namespaces {
+        if !(scope.starts_with("x-") || recognized_namespaces.contains(scope)) {
+            return Err(InvalidRpcIdentifier::UnrecognizedNamespace);
+        }
+    }
+    if !name_ok(name) {
+        return Err(InvalidRpcIdentifier::BadIdName);
+    }
+
+    Ok(())
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -314,4 +353,39 @@ mod test {
             .unwrap();
         assert_eq!(r.v, "cygnets");
     }
+
+    #[test]
+    fn valid_method_names() {
+        let namespaces: HashSet<_> = ["arti", "wombat"].into_iter().collect();
+
+        for name in [
+            "arti:clone",
+            "arti:clone7",
+            "arti:clone_now",
+            "wombat:knish",
+            "x-foo:bar",
+        ] {
+            assert!(is_valid_rpc_identifier(Some(&namespaces), name).is_ok());
+        }
+    }
+
+    #[test]
+    fn invalid_method_names() {
+        let namespaces: HashSet<_> = ["arti", "wombat"].into_iter().collect();
+        use InvalidRpcIdentifier as E;
+
+        for (name, expect_err) in [
+            ("arti-foo:clone", E::UnrecognizedNamespace),
+            ("fred", E::NoNamespace),
+            ("arti:", E::BadIdName),
+            ("arti:7clone", E::BadIdName),
+            ("arti:CLONE", E::BadIdName),
+            ("arti:clone-now", E::BadIdName),
+        ] {
+            assert_eq!(
+                is_valid_rpc_identifier(Some(&namespaces), name),
+                Err(expect_err)
+            );
+        }
+    }
 }
