@@ -1,18 +1,22 @@
 //! Implementation for using `native_tls`
 
-use crate::traits::{CertifiedConn, TlsConnector, TlsProvider};
+use crate::traits::{CertifiedConn, StreamOps, TlsConnector, TlsProvider};
 
 use async_trait::async_trait;
 use futures::{AsyncRead, AsyncWrite};
 use native_tls_crate as native_tls;
 use std::io::{Error as IoError, Result as IoResult};
+use tracing::instrument;
 
 /// A [`TlsProvider`] that uses `native_tls`.
 ///
 /// It supports wrapping any reasonable stream type that implements `AsyncRead` + `AsyncWrite`.
 #[cfg_attr(
     docsrs,
-    doc(cfg(all(feature = "native-tls", any(feature = "tokio", feature = "async-std"))))
+    doc(cfg(all(
+        feature = "native-tls",
+        any(feature = "tokio", feature = "async-std", feature = "smol")
+    )))
 )]
 #[derive(Default, Clone)]
 #[non_exhaustive]
@@ -26,13 +30,11 @@ where
         let cert = self.peer_certificate();
         match cert {
             Ok(Some(c)) => {
-                let der = c
-                    .to_der()
-                    .map_err(|e| IoError::new(std::io::ErrorKind::Other, e))?;
+                let der = c.to_der().map_err(IoError::other)?;
                 Ok(Some(der))
             }
             Ok(None) => Ok(None),
-            Err(e) => Err(IoError::new(std::io::ErrorKind::Other, e)),
+            Err(e) => Err(IoError::other(e)),
         }
     }
 
@@ -49,6 +51,16 @@ where
     }
 }
 
+impl<S: AsyncRead + AsyncWrite + StreamOps + Unpin> StreamOps for async_native_tls::TlsStream<S> {
+    fn set_tcp_notsent_lowat(&self, notsent_lowat: u32) -> IoResult<()> {
+        self.get_ref().set_tcp_notsent_lowat(notsent_lowat)
+    }
+
+    fn new_handle(&self) -> Box<dyn StreamOps + Send + Unpin> {
+        self.get_ref().new_handle()
+    }
+}
+
 /// An implementation of [`TlsConnector`] built with `native_tls`.
 pub struct NativeTlsConnector<S> {
     /// The inner connector object.
@@ -60,23 +72,24 @@ pub struct NativeTlsConnector<S> {
 #[async_trait]
 impl<S> TlsConnector<S> for NativeTlsConnector<S>
 where
-    S: AsyncRead + AsyncWrite + Unpin + Send + 'static,
+    S: AsyncRead + AsyncWrite + StreamOps + Unpin + Send + 'static,
 {
     type Conn = async_native_tls::TlsStream<S>;
 
+    #[instrument(skip_all, level = "trace")]
     async fn negotiate_unvalidated(&self, stream: S, sni_hostname: &str) -> IoResult<Self::Conn> {
         let conn = self
             .connector
             .connect(sni_hostname, stream)
             .await
-            .map_err(|e| IoError::new(std::io::ErrorKind::Other, e))?;
+            .map_err(IoError::other)?;
         Ok(conn)
     }
 }
 
 impl<S> TlsProvider<S> for NativeTlsProvider
 where
-    S: AsyncRead + AsyncWrite + Unpin + Send + 'static,
+    S: AsyncRead + AsyncWrite + StreamOps + Unpin + Send + 'static,
 {
     type Connector = NativeTlsConnector<S>;
 
