@@ -259,12 +259,12 @@ impl<R: SleepProvider> Stream for TaskSchedule<R> {
 #[cfg(all(
     test,
     any(feature = "native-tls", feature = "rustls"),
-    any(feature = "tokio", feature = "async-std"),
+    any(feature = "tokio", feature = "async-std", feature = "smol"),
     not(miri), // Several of these use real SystemTime
 ))]
 mod test {
     use crate::scheduler::TaskSchedule;
-    use crate::{test_with_all_runtimes, SleepProvider};
+    use crate::{SleepProvider, test_with_all_runtimes};
     use futures::FutureExt;
     use futures::StreamExt;
     use std::time::{Duration, Instant};
