@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -34,13 +34,19 @@
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
+#![deny(clippy::mod_module_files)]
 #![allow(clippy::let_unit_value)] // This can reasonably be done for explicitness
 #![allow(clippy::uninlined_format_args)]
 #![allow(clippy::significant_drop_in_scrutinee)] // arti/-/merge_requests/588/#note_2812945
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
+#![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
+// TODO #1645 (either remove this, or decide to have it everywhere)
+#![cfg_attr(any(not(feature = "full"), miri), allow(unused))]
+
 extern crate core;
 
 #[macro_use]
@@ -50,13 +56,23 @@ pub mod io;
 pub mod net;
 pub mod simple_time;
 pub mod task;
+
+// TODO #1885, see MockSleepProvider's cfg_attr deprecated
+//
+// We must use a blanket allow because otherwise every impl we write
+// on the deprecated types would need a separate allow!
+//
+// This is here rather than in time.rs because cfg'd inner attributes don't work properly
+#[cfg_attr(not(test), allow(deprecated))]
 pub mod time;
 
 mod net_runtime;
 mod runtime;
+#[cfg_attr(not(test), allow(deprecated))] // TODO #1885, see comment above on mod time
 mod sleep_runtime;
 mod time_core;
 
 pub use net_runtime::MockNetRuntime;
 pub use runtime::MockRuntime;
+#[allow(deprecated)]
 pub use sleep_runtime::MockSleepRuntime;
